# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmCouponRedemptionEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'coupon': 'OrmCoupon',
        'organization': 'OrmOrganization',
        'user': 'OrmUser'
    }

    attribute_map = {
        'coupon': 'coupon',
        'organization': 'organization',
        'user': 'user'
    }

    def __init__(self, coupon=None, organization=None, user=None, local_vars_configuration=None):  # noqa: E501
        """OrmCouponRedemptionEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._coupon = None
        self._organization = None
        self._user = None
        self.discriminator = None

        if coupon is not None:
            self.coupon = coupon
        if organization is not None:
            self.organization = organization
        if user is not None:
            self.user = user

    @property
    def coupon(self):
        """Gets the coupon of this OrmCouponRedemptionEdges.  # noqa: E501


        :return: The coupon of this OrmCouponRedemptionEdges.  # noqa: E501
        :rtype: OrmCoupon
        """
        return self._coupon

    @coupon.setter
    def coupon(self, coupon):
        """Sets the coupon of this OrmCouponRedemptionEdges.


        :param coupon: The coupon of this OrmCouponRedemptionEdges.  # noqa: E501
        :type coupon: OrmCoupon
        """

        self._coupon = coupon

    @property
    def organization(self):
        """Gets the organization of this OrmCouponRedemptionEdges.  # noqa: E501


        :return: The organization of this OrmCouponRedemptionEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrmCouponRedemptionEdges.


        :param organization: The organization of this OrmCouponRedemptionEdges.  # noqa: E501
        :type organization: OrmOrganization
        """

        self._organization = organization

    @property
    def user(self):
        """Gets the user of this OrmCouponRedemptionEdges.  # noqa: E501


        :return: The user of this OrmCouponRedemptionEdges.  # noqa: E501
        :rtype: OrmUser
        """
        return self._user

    @user.setter
    def user(self, user):
        """Sets the user of this OrmCouponRedemptionEdges.


        :param user: The user of this OrmCouponRedemptionEdges.  # noqa: E501
        :type user: OrmUser
        """

        self._user = user

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmCouponRedemptionEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmCouponRedemptionEdges):
            return True

        return self.to_dict() != other.to_dict()
