# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class SamlIDPSSODescriptor(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'artifact_resolution_services': 'list[SamlEndpoint]',
        'assertion_id_request_services': 'list[SamlEndpoint]',
        'attribute_profiles': 'list[str]',
        'attributes': 'list[SamlAttribute]',
        'cache_duration': 'str',
        'contact_people': 'list[SamlContactPerson]',
        'error_url': 'str',
        'id': 'str',
        'key_descriptors': 'list[SamlKeyDescriptor]',
        'manage_name_id_services': 'list[SamlEndpoint]',
        'name_id_formats': 'list[str]',
        'name_id_mapping_services': 'list[SamlEndpoint]',
        'organization': 'SamlOrganization',
        'protocol_support_enumeration': 'str',
        'signature': 'EtreeElement',
        'single_logout_services': 'list[SamlEndpoint]',
        'single_sign_on_services': 'list[SamlEndpoint]',
        'valid_until': 'datetime',
        'want_authn_requests_signed': 'bool',
        'xml_name': 'XmlName'
    }

    attribute_map = {
        'artifact_resolution_services': 'ArtifactResolutionServices',
        'assertion_id_request_services': 'AssertionIDRequestServices',
        'attribute_profiles': 'AttributeProfiles',
        'attributes': 'Attributes',
        'cache_duration': 'CacheDuration',
        'contact_people': 'ContactPeople',
        'error_url': 'ErrorURL',
        'id': 'ID',
        'key_descriptors': 'KeyDescriptors',
        'manage_name_id_services': 'ManageNameIDServices',
        'name_id_formats': 'NameIDFormats',
        'name_id_mapping_services': 'NameIDMappingServices',
        'organization': 'Organization',
        'protocol_support_enumeration': 'ProtocolSupportEnumeration',
        'signature': 'Signature',
        'single_logout_services': 'SingleLogoutServices',
        'single_sign_on_services': 'SingleSignOnServices',
        'valid_until': 'ValidUntil',
        'want_authn_requests_signed': 'WantAuthnRequestsSigned',
        'xml_name': 'XMLName'
    }

    def __init__(self, artifact_resolution_services=None, assertion_id_request_services=None, attribute_profiles=None, attributes=None, cache_duration=None, contact_people=None, error_url=None, id=None, key_descriptors=None, manage_name_id_services=None, name_id_formats=None, name_id_mapping_services=None, organization=None, protocol_support_enumeration=None, signature=None, single_logout_services=None, single_sign_on_services=None, valid_until=None, want_authn_requests_signed=None, xml_name=None, local_vars_configuration=None):  # noqa: E501
        """SamlIDPSSODescriptor - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._artifact_resolution_services = None
        self._assertion_id_request_services = None
        self._attribute_profiles = None
        self._attributes = None
        self._cache_duration = None
        self._contact_people = None
        self._error_url = None
        self._id = None
        self._key_descriptors = None
        self._manage_name_id_services = None
        self._name_id_formats = None
        self._name_id_mapping_services = None
        self._organization = None
        self._protocol_support_enumeration = None
        self._signature = None
        self._single_logout_services = None
        self._single_sign_on_services = None
        self._valid_until = None
        self._want_authn_requests_signed = None
        self._xml_name = None
        self.discriminator = None

        if artifact_resolution_services is not None:
            self.artifact_resolution_services = artifact_resolution_services
        if assertion_id_request_services is not None:
            self.assertion_id_request_services = assertion_id_request_services
        if attribute_profiles is not None:
            self.attribute_profiles = attribute_profiles
        if attributes is not None:
            self.attributes = attributes
        if cache_duration is not None:
            self.cache_duration = cache_duration
        if contact_people is not None:
            self.contact_people = contact_people
        if error_url is not None:
            self.error_url = error_url
        if id is not None:
            self.id = id
        if key_descriptors is not None:
            self.key_descriptors = key_descriptors
        if manage_name_id_services is not None:
            self.manage_name_id_services = manage_name_id_services
        if name_id_formats is not None:
            self.name_id_formats = name_id_formats
        if name_id_mapping_services is not None:
            self.name_id_mapping_services = name_id_mapping_services
        if organization is not None:
            self.organization = organization
        if protocol_support_enumeration is not None:
            self.protocol_support_enumeration = protocol_support_enumeration
        if signature is not None:
            self.signature = signature
        if single_logout_services is not None:
            self.single_logout_services = single_logout_services
        if single_sign_on_services is not None:
            self.single_sign_on_services = single_sign_on_services
        self.valid_until = valid_until
        self.want_authn_requests_signed = want_authn_requests_signed
        if xml_name is not None:
            self.xml_name = xml_name

    @property
    def artifact_resolution_services(self):
        """Gets the artifact_resolution_services of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The artifact_resolution_services of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlEndpoint]
        """
        return self._artifact_resolution_services

    @artifact_resolution_services.setter
    def artifact_resolution_services(self, artifact_resolution_services):
        """Sets the artifact_resolution_services of this SamlIDPSSODescriptor.


        :param artifact_resolution_services: The artifact_resolution_services of this SamlIDPSSODescriptor.  # noqa: E501
        :type artifact_resolution_services: list[SamlEndpoint]
        """

        self._artifact_resolution_services = artifact_resolution_services

    @property
    def assertion_id_request_services(self):
        """Gets the assertion_id_request_services of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The assertion_id_request_services of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlEndpoint]
        """
        return self._assertion_id_request_services

    @assertion_id_request_services.setter
    def assertion_id_request_services(self, assertion_id_request_services):
        """Sets the assertion_id_request_services of this SamlIDPSSODescriptor.


        :param assertion_id_request_services: The assertion_id_request_services of this SamlIDPSSODescriptor.  # noqa: E501
        :type assertion_id_request_services: list[SamlEndpoint]
        """

        self._assertion_id_request_services = assertion_id_request_services

    @property
    def attribute_profiles(self):
        """Gets the attribute_profiles of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The attribute_profiles of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[str]
        """
        return self._attribute_profiles

    @attribute_profiles.setter
    def attribute_profiles(self, attribute_profiles):
        """Sets the attribute_profiles of this SamlIDPSSODescriptor.


        :param attribute_profiles: The attribute_profiles of this SamlIDPSSODescriptor.  # noqa: E501
        :type attribute_profiles: list[str]
        """

        self._attribute_profiles = attribute_profiles

    @property
    def attributes(self):
        """Gets the attributes of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The attributes of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlAttribute]
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """Sets the attributes of this SamlIDPSSODescriptor.


        :param attributes: The attributes of this SamlIDPSSODescriptor.  # noqa: E501
        :type attributes: list[SamlAttribute]
        """

        self._attributes = attributes

    @property
    def cache_duration(self):
        """Gets the cache_duration of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The cache_duration of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: str
        """
        return self._cache_duration

    @cache_duration.setter
    def cache_duration(self, cache_duration):
        """Sets the cache_duration of this SamlIDPSSODescriptor.


        :param cache_duration: The cache_duration of this SamlIDPSSODescriptor.  # noqa: E501
        :type cache_duration: str
        """

        self._cache_duration = cache_duration

    @property
    def contact_people(self):
        """Gets the contact_people of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The contact_people of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlContactPerson]
        """
        return self._contact_people

    @contact_people.setter
    def contact_people(self, contact_people):
        """Sets the contact_people of this SamlIDPSSODescriptor.


        :param contact_people: The contact_people of this SamlIDPSSODescriptor.  # noqa: E501
        :type contact_people: list[SamlContactPerson]
        """

        self._contact_people = contact_people

    @property
    def error_url(self):
        """Gets the error_url of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The error_url of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: str
        """
        return self._error_url

    @error_url.setter
    def error_url(self, error_url):
        """Sets the error_url of this SamlIDPSSODescriptor.


        :param error_url: The error_url of this SamlIDPSSODescriptor.  # noqa: E501
        :type error_url: str
        """

        self._error_url = error_url

    @property
    def id(self):
        """Gets the id of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The id of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SamlIDPSSODescriptor.


        :param id: The id of this SamlIDPSSODescriptor.  # noqa: E501
        :type id: str
        """

        self._id = id

    @property
    def key_descriptors(self):
        """Gets the key_descriptors of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The key_descriptors of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlKeyDescriptor]
        """
        return self._key_descriptors

    @key_descriptors.setter
    def key_descriptors(self, key_descriptors):
        """Sets the key_descriptors of this SamlIDPSSODescriptor.


        :param key_descriptors: The key_descriptors of this SamlIDPSSODescriptor.  # noqa: E501
        :type key_descriptors: list[SamlKeyDescriptor]
        """

        self._key_descriptors = key_descriptors

    @property
    def manage_name_id_services(self):
        """Gets the manage_name_id_services of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The manage_name_id_services of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlEndpoint]
        """
        return self._manage_name_id_services

    @manage_name_id_services.setter
    def manage_name_id_services(self, manage_name_id_services):
        """Sets the manage_name_id_services of this SamlIDPSSODescriptor.


        :param manage_name_id_services: The manage_name_id_services of this SamlIDPSSODescriptor.  # noqa: E501
        :type manage_name_id_services: list[SamlEndpoint]
        """

        self._manage_name_id_services = manage_name_id_services

    @property
    def name_id_formats(self):
        """Gets the name_id_formats of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The name_id_formats of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[str]
        """
        return self._name_id_formats

    @name_id_formats.setter
    def name_id_formats(self, name_id_formats):
        """Sets the name_id_formats of this SamlIDPSSODescriptor.


        :param name_id_formats: The name_id_formats of this SamlIDPSSODescriptor.  # noqa: E501
        :type name_id_formats: list[str]
        """

        self._name_id_formats = name_id_formats

    @property
    def name_id_mapping_services(self):
        """Gets the name_id_mapping_services of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The name_id_mapping_services of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlEndpoint]
        """
        return self._name_id_mapping_services

    @name_id_mapping_services.setter
    def name_id_mapping_services(self, name_id_mapping_services):
        """Sets the name_id_mapping_services of this SamlIDPSSODescriptor.


        :param name_id_mapping_services: The name_id_mapping_services of this SamlIDPSSODescriptor.  # noqa: E501
        :type name_id_mapping_services: list[SamlEndpoint]
        """

        self._name_id_mapping_services = name_id_mapping_services

    @property
    def organization(self):
        """Gets the organization of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The organization of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: SamlOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this SamlIDPSSODescriptor.


        :param organization: The organization of this SamlIDPSSODescriptor.  # noqa: E501
        :type organization: SamlOrganization
        """

        self._organization = organization

    @property
    def protocol_support_enumeration(self):
        """Gets the protocol_support_enumeration of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The protocol_support_enumeration of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: str
        """
        return self._protocol_support_enumeration

    @protocol_support_enumeration.setter
    def protocol_support_enumeration(self, protocol_support_enumeration):
        """Sets the protocol_support_enumeration of this SamlIDPSSODescriptor.


        :param protocol_support_enumeration: The protocol_support_enumeration of this SamlIDPSSODescriptor.  # noqa: E501
        :type protocol_support_enumeration: str
        """

        self._protocol_support_enumeration = protocol_support_enumeration

    @property
    def signature(self):
        """Gets the signature of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The signature of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: EtreeElement
        """
        return self._signature

    @signature.setter
    def signature(self, signature):
        """Sets the signature of this SamlIDPSSODescriptor.


        :param signature: The signature of this SamlIDPSSODescriptor.  # noqa: E501
        :type signature: EtreeElement
        """

        self._signature = signature

    @property
    def single_logout_services(self):
        """Gets the single_logout_services of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The single_logout_services of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlEndpoint]
        """
        return self._single_logout_services

    @single_logout_services.setter
    def single_logout_services(self, single_logout_services):
        """Sets the single_logout_services of this SamlIDPSSODescriptor.


        :param single_logout_services: The single_logout_services of this SamlIDPSSODescriptor.  # noqa: E501
        :type single_logout_services: list[SamlEndpoint]
        """

        self._single_logout_services = single_logout_services

    @property
    def single_sign_on_services(self):
        """Gets the single_sign_on_services of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The single_sign_on_services of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: list[SamlEndpoint]
        """
        return self._single_sign_on_services

    @single_sign_on_services.setter
    def single_sign_on_services(self, single_sign_on_services):
        """Sets the single_sign_on_services of this SamlIDPSSODescriptor.


        :param single_sign_on_services: The single_sign_on_services of this SamlIDPSSODescriptor.  # noqa: E501
        :type single_sign_on_services: list[SamlEndpoint]
        """

        self._single_sign_on_services = single_sign_on_services

    @property
    def valid_until(self):
        """Gets the valid_until of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The valid_until of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: datetime
        """
        return self._valid_until

    @valid_until.setter
    def valid_until(self, valid_until):
        """Sets the valid_until of this SamlIDPSSODescriptor.


        :param valid_until: The valid_until of this SamlIDPSSODescriptor.  # noqa: E501
        :type valid_until: datetime
        """

        self._valid_until = valid_until

    @property
    def want_authn_requests_signed(self):
        """Gets the want_authn_requests_signed of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The want_authn_requests_signed of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: bool
        """
        return self._want_authn_requests_signed

    @want_authn_requests_signed.setter
    def want_authn_requests_signed(self, want_authn_requests_signed):
        """Sets the want_authn_requests_signed of this SamlIDPSSODescriptor.


        :param want_authn_requests_signed: The want_authn_requests_signed of this SamlIDPSSODescriptor.  # noqa: E501
        :type want_authn_requests_signed: bool
        """

        self._want_authn_requests_signed = want_authn_requests_signed

    @property
    def xml_name(self):
        """Gets the xml_name of this SamlIDPSSODescriptor.  # noqa: E501


        :return: The xml_name of this SamlIDPSSODescriptor.  # noqa: E501
        :rtype: XmlName
        """
        return self._xml_name

    @xml_name.setter
    def xml_name(self, xml_name):
        """Sets the xml_name of this SamlIDPSSODescriptor.


        :param xml_name: The xml_name of this SamlIDPSSODescriptor.  # noqa: E501
        :type xml_name: XmlName
        """

        self._xml_name = xml_name

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SamlIDPSSODescriptor):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SamlIDPSSODescriptor):
            return True

        return self.to_dict() != other.to_dict()
