# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmLLMDocument(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'artifact_id': 'int',
        'chunking_method_params': 'dict[str, object]',
        'chunking_method_type': 'str',
        'created_dt': 'datetime',
        'download_url': 'str',
        'edges': 'OrmLLMDocumentEdges',
        'extension': 'str',
        'filename': 'str',
        'id': 'int',
        'llm_knowledge_id': 'int',
        'organization_id': 'int',
        'parser_type': 'str',
        'size': 'str',
        'slug': 'str',
        'updated_dt': 'datetime',
        'upload_successful': 'bool',
        'uploaded_by_id': 'int'
    }

    attribute_map = {
        'artifact_id': 'artifact_id',
        'chunking_method_params': 'chunking_method_params',
        'chunking_method_type': 'chunking_method_type',
        'created_dt': 'created_dt',
        'download_url': 'download_url',
        'edges': 'edges',
        'extension': 'extension',
        'filename': 'filename',
        'id': 'id',
        'llm_knowledge_id': 'llm_knowledge_id',
        'organization_id': 'organization_id',
        'parser_type': 'parser_type',
        'size': 'size',
        'slug': 'slug',
        'updated_dt': 'updated_dt',
        'upload_successful': 'upload_successful',
        'uploaded_by_id': 'uploaded_by_id'
    }

    def __init__(self, artifact_id=None, chunking_method_params=None, chunking_method_type=None, created_dt=None, download_url=None, edges=None, extension=None, filename=None, id=None, llm_knowledge_id=None, organization_id=None, parser_type=None, size=None, slug=None, updated_dt=None, upload_successful=None, uploaded_by_id=None, local_vars_configuration=None):  # noqa: E501
        """OrmLLMDocument - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._artifact_id = None
        self._chunking_method_params = None
        self._chunking_method_type = None
        self._created_dt = None
        self._download_url = None
        self._edges = None
        self._extension = None
        self._filename = None
        self._id = None
        self._llm_knowledge_id = None
        self._organization_id = None
        self._parser_type = None
        self._size = None
        self._slug = None
        self._updated_dt = None
        self._upload_successful = None
        self._uploaded_by_id = None
        self.discriminator = None

        if artifact_id is not None:
            self.artifact_id = artifact_id
        if chunking_method_params is not None:
            self.chunking_method_params = chunking_method_params
        if chunking_method_type is not None:
            self.chunking_method_type = chunking_method_type
        if created_dt is not None:
            self.created_dt = created_dt
        if download_url is not None:
            self.download_url = download_url
        if edges is not None:
            self.edges = edges
        if extension is not None:
            self.extension = extension
        if filename is not None:
            self.filename = filename
        if id is not None:
            self.id = id
        if llm_knowledge_id is not None:
            self.llm_knowledge_id = llm_knowledge_id
        if organization_id is not None:
            self.organization_id = organization_id
        if parser_type is not None:
            self.parser_type = parser_type
        if size is not None:
            self.size = size
        if slug is not None:
            self.slug = slug
        if updated_dt is not None:
            self.updated_dt = updated_dt
        if upload_successful is not None:
            self.upload_successful = upload_successful
        if uploaded_by_id is not None:
            self.uploaded_by_id = uploaded_by_id

    @property
    def artifact_id(self):
        """Gets the artifact_id of this OrmLLMDocument.  # noqa: E501


        :return: The artifact_id of this OrmLLMDocument.  # noqa: E501
        :rtype: int
        """
        return self._artifact_id

    @artifact_id.setter
    def artifact_id(self, artifact_id):
        """Sets the artifact_id of this OrmLLMDocument.


        :param artifact_id: The artifact_id of this OrmLLMDocument.  # noqa: E501
        :type artifact_id: int
        """

        self._artifact_id = artifact_id

    @property
    def chunking_method_params(self):
        """Gets the chunking_method_params of this OrmLLMDocument.  # noqa: E501


        :return: The chunking_method_params of this OrmLLMDocument.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._chunking_method_params

    @chunking_method_params.setter
    def chunking_method_params(self, chunking_method_params):
        """Sets the chunking_method_params of this OrmLLMDocument.


        :param chunking_method_params: The chunking_method_params of this OrmLLMDocument.  # noqa: E501
        :type chunking_method_params: dict[str, object]
        """

        self._chunking_method_params = chunking_method_params

    @property
    def chunking_method_type(self):
        """Gets the chunking_method_type of this OrmLLMDocument.  # noqa: E501


        :return: The chunking_method_type of this OrmLLMDocument.  # noqa: E501
        :rtype: str
        """
        return self._chunking_method_type

    @chunking_method_type.setter
    def chunking_method_type(self, chunking_method_type):
        """Sets the chunking_method_type of this OrmLLMDocument.


        :param chunking_method_type: The chunking_method_type of this OrmLLMDocument.  # noqa: E501
        :type chunking_method_type: str
        """

        self._chunking_method_type = chunking_method_type

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmLLMDocument.  # noqa: E501


        :return: The created_dt of this OrmLLMDocument.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmLLMDocument.


        :param created_dt: The created_dt of this OrmLLMDocument.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def download_url(self):
        """Gets the download_url of this OrmLLMDocument.  # noqa: E501


        :return: The download_url of this OrmLLMDocument.  # noqa: E501
        :rtype: str
        """
        return self._download_url

    @download_url.setter
    def download_url(self, download_url):
        """Sets the download_url of this OrmLLMDocument.


        :param download_url: The download_url of this OrmLLMDocument.  # noqa: E501
        :type download_url: str
        """

        self._download_url = download_url

    @property
    def edges(self):
        """Gets the edges of this OrmLLMDocument.  # noqa: E501


        :return: The edges of this OrmLLMDocument.  # noqa: E501
        :rtype: OrmLLMDocumentEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmLLMDocument.


        :param edges: The edges of this OrmLLMDocument.  # noqa: E501
        :type edges: OrmLLMDocumentEdges
        """

        self._edges = edges

    @property
    def extension(self):
        """Gets the extension of this OrmLLMDocument.  # noqa: E501


        :return: The extension of this OrmLLMDocument.  # noqa: E501
        :rtype: str
        """
        return self._extension

    @extension.setter
    def extension(self, extension):
        """Sets the extension of this OrmLLMDocument.


        :param extension: The extension of this OrmLLMDocument.  # noqa: E501
        :type extension: str
        """

        self._extension = extension

    @property
    def filename(self):
        """Gets the filename of this OrmLLMDocument.  # noqa: E501


        :return: The filename of this OrmLLMDocument.  # noqa: E501
        :rtype: str
        """
        return self._filename

    @filename.setter
    def filename(self, filename):
        """Sets the filename of this OrmLLMDocument.


        :param filename: The filename of this OrmLLMDocument.  # noqa: E501
        :type filename: str
        """

        self._filename = filename

    @property
    def id(self):
        """Gets the id of this OrmLLMDocument.  # noqa: E501


        :return: The id of this OrmLLMDocument.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmLLMDocument.


        :param id: The id of this OrmLLMDocument.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def llm_knowledge_id(self):
        """Gets the llm_knowledge_id of this OrmLLMDocument.  # noqa: E501


        :return: The llm_knowledge_id of this OrmLLMDocument.  # noqa: E501
        :rtype: int
        """
        return self._llm_knowledge_id

    @llm_knowledge_id.setter
    def llm_knowledge_id(self, llm_knowledge_id):
        """Sets the llm_knowledge_id of this OrmLLMDocument.


        :param llm_knowledge_id: The llm_knowledge_id of this OrmLLMDocument.  # noqa: E501
        :type llm_knowledge_id: int
        """

        self._llm_knowledge_id = llm_knowledge_id

    @property
    def organization_id(self):
        """Gets the organization_id of this OrmLLMDocument.  # noqa: E501


        :return: The organization_id of this OrmLLMDocument.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrmLLMDocument.


        :param organization_id: The organization_id of this OrmLLMDocument.  # noqa: E501
        :type organization_id: int
        """

        self._organization_id = organization_id

    @property
    def parser_type(self):
        """Gets the parser_type of this OrmLLMDocument.  # noqa: E501


        :return: The parser_type of this OrmLLMDocument.  # noqa: E501
        :rtype: str
        """
        return self._parser_type

    @parser_type.setter
    def parser_type(self, parser_type):
        """Sets the parser_type of this OrmLLMDocument.


        :param parser_type: The parser_type of this OrmLLMDocument.  # noqa: E501
        :type parser_type: str
        """

        self._parser_type = parser_type

    @property
    def size(self):
        """Gets the size of this OrmLLMDocument.  # noqa: E501


        :return: The size of this OrmLLMDocument.  # noqa: E501
        :rtype: str
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this OrmLLMDocument.


        :param size: The size of this OrmLLMDocument.  # noqa: E501
        :type size: str
        """

        self._size = size

    @property
    def slug(self):
        """Gets the slug of this OrmLLMDocument.  # noqa: E501


        :return: The slug of this OrmLLMDocument.  # noqa: E501
        :rtype: str
        """
        return self._slug

    @slug.setter
    def slug(self, slug):
        """Sets the slug of this OrmLLMDocument.


        :param slug: The slug of this OrmLLMDocument.  # noqa: E501
        :type slug: str
        """

        self._slug = slug

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmLLMDocument.  # noqa: E501


        :return: The updated_dt of this OrmLLMDocument.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmLLMDocument.


        :param updated_dt: The updated_dt of this OrmLLMDocument.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def upload_successful(self):
        """Gets the upload_successful of this OrmLLMDocument.  # noqa: E501


        :return: The upload_successful of this OrmLLMDocument.  # noqa: E501
        :rtype: bool
        """
        return self._upload_successful

    @upload_successful.setter
    def upload_successful(self, upload_successful):
        """Sets the upload_successful of this OrmLLMDocument.


        :param upload_successful: The upload_successful of this OrmLLMDocument.  # noqa: E501
        :type upload_successful: bool
        """

        self._upload_successful = upload_successful

    @property
    def uploaded_by_id(self):
        """Gets the uploaded_by_id of this OrmLLMDocument.  # noqa: E501


        :return: The uploaded_by_id of this OrmLLMDocument.  # noqa: E501
        :rtype: int
        """
        return self._uploaded_by_id

    @uploaded_by_id.setter
    def uploaded_by_id(self, uploaded_by_id):
        """Sets the uploaded_by_id of this OrmLLMDocument.


        :param uploaded_by_id: The uploaded_by_id of this OrmLLMDocument.  # noqa: E501
        :type uploaded_by_id: int
        """

        self._uploaded_by_id = uploaded_by_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmLLMDocument):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmLLMDocument):
            return True

        return self.to_dict() != other.to_dict()
