# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class ResponseProjectInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'description': 'str',
        'experiment_summary': 'ResponseProjectExperimentSummary',
        'id': 'int',
        'is_example': 'bool',
        'key_metrics': 'OrmKeyMetrics',
        'name': 'str',
        'note': 'str',
        'organization': 'ResponseOrganization',
        'primary_owner': 'ResponseUser',
        'project_datasets': 'list[ResponseProjectDataset]',
        'project_repositories': 'list[ResponseProjectRepository]',
        'run_execution_summary': 'ResponseProjectRunExecutionSummary',
        'starred': 'bool',
        'sweep_summary': 'ResponseProjectSweepSummary',
        'updated_dt': 'datetime',
        'volume_id': 'int'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'description': 'description',
        'experiment_summary': 'experiment_summary',
        'id': 'id',
        'is_example': 'is_example',
        'key_metrics': 'key_metrics',
        'name': 'name',
        'note': 'note',
        'organization': 'organization',
        'primary_owner': 'primary_owner',
        'project_datasets': 'project_datasets',
        'project_repositories': 'project_repositories',
        'run_execution_summary': 'run_execution_summary',
        'starred': 'starred',
        'sweep_summary': 'sweep_summary',
        'updated_dt': 'updated_dt',
        'volume_id': 'volume_id'
    }

    def __init__(self, created_dt=None, description=None, experiment_summary=None, id=None, is_example=None, key_metrics=None, name=None, note=None, organization=None, primary_owner=None, project_datasets=None, project_repositories=None, run_execution_summary=None, starred=None, sweep_summary=None, updated_dt=None, volume_id=None, local_vars_configuration=None):  # noqa: E501
        """ResponseProjectInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._description = None
        self._experiment_summary = None
        self._id = None
        self._is_example = None
        self._key_metrics = None
        self._name = None
        self._note = None
        self._organization = None
        self._primary_owner = None
        self._project_datasets = None
        self._project_repositories = None
        self._run_execution_summary = None
        self._starred = None
        self._sweep_summary = None
        self._updated_dt = None
        self._volume_id = None
        self.discriminator = None

        self.created_dt = created_dt
        self.description = description
        self.experiment_summary = experiment_summary
        self.id = id
        self.is_example = is_example
        if key_metrics is not None:
            self.key_metrics = key_metrics
        self.name = name
        self.note = note
        self.organization = organization
        self.primary_owner = primary_owner
        if project_datasets is not None:
            self.project_datasets = project_datasets
        if project_repositories is not None:
            self.project_repositories = project_repositories
        if run_execution_summary is not None:
            self.run_execution_summary = run_execution_summary
        self.starred = starred
        if sweep_summary is not None:
            self.sweep_summary = sweep_summary
        self.updated_dt = updated_dt
        self.volume_id = volume_id

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseProjectInfo.  # noqa: E501


        :return: The created_dt of this ResponseProjectInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseProjectInfo.


        :param created_dt: The created_dt of this ResponseProjectInfo.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def description(self):
        """Gets the description of this ResponseProjectInfo.  # noqa: E501


        :return: The description of this ResponseProjectInfo.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ResponseProjectInfo.


        :param description: The description of this ResponseProjectInfo.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def experiment_summary(self):
        """Gets the experiment_summary of this ResponseProjectInfo.  # noqa: E501


        :return: The experiment_summary of this ResponseProjectInfo.  # noqa: E501
        :rtype: ResponseProjectExperimentSummary
        """
        return self._experiment_summary

    @experiment_summary.setter
    def experiment_summary(self, experiment_summary):
        """Sets the experiment_summary of this ResponseProjectInfo.


        :param experiment_summary: The experiment_summary of this ResponseProjectInfo.  # noqa: E501
        :type experiment_summary: ResponseProjectExperimentSummary
        """
        if self.local_vars_configuration.client_side_validation and experiment_summary is None:  # noqa: E501
            raise ValueError("Invalid value for `experiment_summary`, must not be `None`")  # noqa: E501

        self._experiment_summary = experiment_summary

    @property
    def id(self):
        """Gets the id of this ResponseProjectInfo.  # noqa: E501


        :return: The id of this ResponseProjectInfo.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseProjectInfo.


        :param id: The id of this ResponseProjectInfo.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def is_example(self):
        """Gets the is_example of this ResponseProjectInfo.  # noqa: E501


        :return: The is_example of this ResponseProjectInfo.  # noqa: E501
        :rtype: bool
        """
        return self._is_example

    @is_example.setter
    def is_example(self, is_example):
        """Sets the is_example of this ResponseProjectInfo.


        :param is_example: The is_example of this ResponseProjectInfo.  # noqa: E501
        :type is_example: bool
        """
        if self.local_vars_configuration.client_side_validation and is_example is None:  # noqa: E501
            raise ValueError("Invalid value for `is_example`, must not be `None`")  # noqa: E501

        self._is_example = is_example

    @property
    def key_metrics(self):
        """Gets the key_metrics of this ResponseProjectInfo.  # noqa: E501


        :return: The key_metrics of this ResponseProjectInfo.  # noqa: E501
        :rtype: OrmKeyMetrics
        """
        return self._key_metrics

    @key_metrics.setter
    def key_metrics(self, key_metrics):
        """Sets the key_metrics of this ResponseProjectInfo.


        :param key_metrics: The key_metrics of this ResponseProjectInfo.  # noqa: E501
        :type key_metrics: OrmKeyMetrics
        """

        self._key_metrics = key_metrics

    @property
    def name(self):
        """Gets the name of this ResponseProjectInfo.  # noqa: E501


        :return: The name of this ResponseProjectInfo.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponseProjectInfo.


        :param name: The name of this ResponseProjectInfo.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def note(self):
        """Gets the note of this ResponseProjectInfo.  # noqa: E501


        :return: The note of this ResponseProjectInfo.  # noqa: E501
        :rtype: str
        """
        return self._note

    @note.setter
    def note(self, note):
        """Sets the note of this ResponseProjectInfo.


        :param note: The note of this ResponseProjectInfo.  # noqa: E501
        :type note: str
        """

        self._note = note

    @property
    def organization(self):
        """Gets the organization of this ResponseProjectInfo.  # noqa: E501


        :return: The organization of this ResponseProjectInfo.  # noqa: E501
        :rtype: ResponseOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this ResponseProjectInfo.


        :param organization: The organization of this ResponseProjectInfo.  # noqa: E501
        :type organization: ResponseOrganization
        """
        if self.local_vars_configuration.client_side_validation and organization is None:  # noqa: E501
            raise ValueError("Invalid value for `organization`, must not be `None`")  # noqa: E501

        self._organization = organization

    @property
    def primary_owner(self):
        """Gets the primary_owner of this ResponseProjectInfo.  # noqa: E501


        :return: The primary_owner of this ResponseProjectInfo.  # noqa: E501
        :rtype: ResponseUser
        """
        return self._primary_owner

    @primary_owner.setter
    def primary_owner(self, primary_owner):
        """Sets the primary_owner of this ResponseProjectInfo.


        :param primary_owner: The primary_owner of this ResponseProjectInfo.  # noqa: E501
        :type primary_owner: ResponseUser
        """
        if self.local_vars_configuration.client_side_validation and primary_owner is None:  # noqa: E501
            raise ValueError("Invalid value for `primary_owner`, must not be `None`")  # noqa: E501

        self._primary_owner = primary_owner

    @property
    def project_datasets(self):
        """Gets the project_datasets of this ResponseProjectInfo.  # noqa: E501


        :return: The project_datasets of this ResponseProjectInfo.  # noqa: E501
        :rtype: list[ResponseProjectDataset]
        """
        return self._project_datasets

    @project_datasets.setter
    def project_datasets(self, project_datasets):
        """Sets the project_datasets of this ResponseProjectInfo.


        :param project_datasets: The project_datasets of this ResponseProjectInfo.  # noqa: E501
        :type project_datasets: list[ResponseProjectDataset]
        """

        self._project_datasets = project_datasets

    @property
    def project_repositories(self):
        """Gets the project_repositories of this ResponseProjectInfo.  # noqa: E501


        :return: The project_repositories of this ResponseProjectInfo.  # noqa: E501
        :rtype: list[ResponseProjectRepository]
        """
        return self._project_repositories

    @project_repositories.setter
    def project_repositories(self, project_repositories):
        """Sets the project_repositories of this ResponseProjectInfo.


        :param project_repositories: The project_repositories of this ResponseProjectInfo.  # noqa: E501
        :type project_repositories: list[ResponseProjectRepository]
        """

        self._project_repositories = project_repositories

    @property
    def run_execution_summary(self):
        """Gets the run_execution_summary of this ResponseProjectInfo.  # noqa: E501


        :return: The run_execution_summary of this ResponseProjectInfo.  # noqa: E501
        :rtype: ResponseProjectRunExecutionSummary
        """
        return self._run_execution_summary

    @run_execution_summary.setter
    def run_execution_summary(self, run_execution_summary):
        """Sets the run_execution_summary of this ResponseProjectInfo.


        :param run_execution_summary: The run_execution_summary of this ResponseProjectInfo.  # noqa: E501
        :type run_execution_summary: ResponseProjectRunExecutionSummary
        """

        self._run_execution_summary = run_execution_summary

    @property
    def starred(self):
        """Gets the starred of this ResponseProjectInfo.  # noqa: E501


        :return: The starred of this ResponseProjectInfo.  # noqa: E501
        :rtype: bool
        """
        return self._starred

    @starred.setter
    def starred(self, starred):
        """Sets the starred of this ResponseProjectInfo.


        :param starred: The starred of this ResponseProjectInfo.  # noqa: E501
        :type starred: bool
        """
        if self.local_vars_configuration.client_side_validation and starred is None:  # noqa: E501
            raise ValueError("Invalid value for `starred`, must not be `None`")  # noqa: E501

        self._starred = starred

    @property
    def sweep_summary(self):
        """Gets the sweep_summary of this ResponseProjectInfo.  # noqa: E501


        :return: The sweep_summary of this ResponseProjectInfo.  # noqa: E501
        :rtype: ResponseProjectSweepSummary
        """
        return self._sweep_summary

    @sweep_summary.setter
    def sweep_summary(self, sweep_summary):
        """Sets the sweep_summary of this ResponseProjectInfo.


        :param sweep_summary: The sweep_summary of this ResponseProjectInfo.  # noqa: E501
        :type sweep_summary: ResponseProjectSweepSummary
        """

        self._sweep_summary = sweep_summary

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseProjectInfo.  # noqa: E501


        :return: The updated_dt of this ResponseProjectInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseProjectInfo.


        :param updated_dt: The updated_dt of this ResponseProjectInfo.  # noqa: E501
        :type updated_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_dt`, must not be `None`")  # noqa: E501

        self._updated_dt = updated_dt

    @property
    def volume_id(self):
        """Gets the volume_id of this ResponseProjectInfo.  # noqa: E501


        :return: The volume_id of this ResponseProjectInfo.  # noqa: E501
        :rtype: int
        """
        return self._volume_id

    @volume_id.setter
    def volume_id(self, volume_id):
        """Sets the volume_id of this ResponseProjectInfo.


        :param volume_id: The volume_id of this ResponseProjectInfo.  # noqa: E501
        :type volume_id: int
        """
        if self.local_vars_configuration.client_side_validation and volume_id is None:  # noqa: E501
            raise ValueError("Invalid value for `volume_id`, must not be `None`")  # noqa: E501

        self._volume_id = volume_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseProjectInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseProjectInfo):
            return True

        return self.to_dict() != other.to_dict()
