# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmVolumeV2(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_by_user_id': 'int',
        'created_by_workload_id': 'int',
        'created_dt': 'datetime',
        'edges': 'OrmVolumeV2Edges',
        'id': 'int',
        'immutable_slug': 'str',
        'is_auto_created': 'bool',
        'is_read_only': 'bool',
        'name': 'str',
        'origin_filesystem_path': 'str',
        'storage_id': 'int',
        'sub_path': 'str',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'created_by_user_id': 'created_by_user_id',
        'created_by_workload_id': 'created_by_workload_id',
        'created_dt': 'created_dt',
        'edges': 'edges',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'is_auto_created': 'is_auto_created',
        'is_read_only': 'is_read_only',
        'name': 'name',
        'origin_filesystem_path': 'origin_filesystem_path',
        'storage_id': 'storage_id',
        'sub_path': 'sub_path',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, created_by_user_id=None, created_by_workload_id=None, created_dt=None, edges=None, id=None, immutable_slug=None, is_auto_created=None, is_read_only=None, name=None, origin_filesystem_path=None, storage_id=None, sub_path=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """OrmVolumeV2 - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_by_user_id = None
        self._created_by_workload_id = None
        self._created_dt = None
        self._edges = None
        self._id = None
        self._immutable_slug = None
        self._is_auto_created = None
        self._is_read_only = None
        self._name = None
        self._origin_filesystem_path = None
        self._storage_id = None
        self._sub_path = None
        self._updated_dt = None
        self.discriminator = None

        self.created_by_user_id = created_by_user_id
        self.created_by_workload_id = created_by_workload_id
        if created_dt is not None:
            self.created_dt = created_dt
        if edges is not None:
            self.edges = edges
        if id is not None:
            self.id = id
        if immutable_slug is not None:
            self.immutable_slug = immutable_slug
        if is_auto_created is not None:
            self.is_auto_created = is_auto_created
        if is_read_only is not None:
            self.is_read_only = is_read_only
        if name is not None:
            self.name = name
        self.origin_filesystem_path = origin_filesystem_path
        if storage_id is not None:
            self.storage_id = storage_id
        if sub_path is not None:
            self.sub_path = sub_path
        if updated_dt is not None:
            self.updated_dt = updated_dt

    @property
    def created_by_user_id(self):
        """Gets the created_by_user_id of this OrmVolumeV2.  # noqa: E501


        :return: The created_by_user_id of this OrmVolumeV2.  # noqa: E501
        :rtype: int
        """
        return self._created_by_user_id

    @created_by_user_id.setter
    def created_by_user_id(self, created_by_user_id):
        """Sets the created_by_user_id of this OrmVolumeV2.


        :param created_by_user_id: The created_by_user_id of this OrmVolumeV2.  # noqa: E501
        :type created_by_user_id: int
        """

        self._created_by_user_id = created_by_user_id

    @property
    def created_by_workload_id(self):
        """Gets the created_by_workload_id of this OrmVolumeV2.  # noqa: E501


        :return: The created_by_workload_id of this OrmVolumeV2.  # noqa: E501
        :rtype: int
        """
        return self._created_by_workload_id

    @created_by_workload_id.setter
    def created_by_workload_id(self, created_by_workload_id):
        """Sets the created_by_workload_id of this OrmVolumeV2.


        :param created_by_workload_id: The created_by_workload_id of this OrmVolumeV2.  # noqa: E501
        :type created_by_workload_id: int
        """

        self._created_by_workload_id = created_by_workload_id

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmVolumeV2.  # noqa: E501


        :return: The created_dt of this OrmVolumeV2.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmVolumeV2.


        :param created_dt: The created_dt of this OrmVolumeV2.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def edges(self):
        """Gets the edges of this OrmVolumeV2.  # noqa: E501


        :return: The edges of this OrmVolumeV2.  # noqa: E501
        :rtype: OrmVolumeV2Edges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmVolumeV2.


        :param edges: The edges of this OrmVolumeV2.  # noqa: E501
        :type edges: OrmVolumeV2Edges
        """

        self._edges = edges

    @property
    def id(self):
        """Gets the id of this OrmVolumeV2.  # noqa: E501


        :return: The id of this OrmVolumeV2.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmVolumeV2.


        :param id: The id of this OrmVolumeV2.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this OrmVolumeV2.  # noqa: E501


        :return: The immutable_slug of this OrmVolumeV2.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this OrmVolumeV2.


        :param immutable_slug: The immutable_slug of this OrmVolumeV2.  # noqa: E501
        :type immutable_slug: str
        """

        self._immutable_slug = immutable_slug

    @property
    def is_auto_created(self):
        """Gets the is_auto_created of this OrmVolumeV2.  # noqa: E501


        :return: The is_auto_created of this OrmVolumeV2.  # noqa: E501
        :rtype: bool
        """
        return self._is_auto_created

    @is_auto_created.setter
    def is_auto_created(self, is_auto_created):
        """Sets the is_auto_created of this OrmVolumeV2.


        :param is_auto_created: The is_auto_created of this OrmVolumeV2.  # noqa: E501
        :type is_auto_created: bool
        """

        self._is_auto_created = is_auto_created

    @property
    def is_read_only(self):
        """Gets the is_read_only of this OrmVolumeV2.  # noqa: E501


        :return: The is_read_only of this OrmVolumeV2.  # noqa: E501
        :rtype: bool
        """
        return self._is_read_only

    @is_read_only.setter
    def is_read_only(self, is_read_only):
        """Sets the is_read_only of this OrmVolumeV2.


        :param is_read_only: The is_read_only of this OrmVolumeV2.  # noqa: E501
        :type is_read_only: bool
        """

        self._is_read_only = is_read_only

    @property
    def name(self):
        """Gets the name of this OrmVolumeV2.  # noqa: E501


        :return: The name of this OrmVolumeV2.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrmVolumeV2.


        :param name: The name of this OrmVolumeV2.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def origin_filesystem_path(self):
        """Gets the origin_filesystem_path of this OrmVolumeV2.  # noqa: E501


        :return: The origin_filesystem_path of this OrmVolumeV2.  # noqa: E501
        :rtype: str
        """
        return self._origin_filesystem_path

    @origin_filesystem_path.setter
    def origin_filesystem_path(self, origin_filesystem_path):
        """Sets the origin_filesystem_path of this OrmVolumeV2.


        :param origin_filesystem_path: The origin_filesystem_path of this OrmVolumeV2.  # noqa: E501
        :type origin_filesystem_path: str
        """

        self._origin_filesystem_path = origin_filesystem_path

    @property
    def storage_id(self):
        """Gets the storage_id of this OrmVolumeV2.  # noqa: E501


        :return: The storage_id of this OrmVolumeV2.  # noqa: E501
        :rtype: int
        """
        return self._storage_id

    @storage_id.setter
    def storage_id(self, storage_id):
        """Sets the storage_id of this OrmVolumeV2.


        :param storage_id: The storage_id of this OrmVolumeV2.  # noqa: E501
        :type storage_id: int
        """

        self._storage_id = storage_id

    @property
    def sub_path(self):
        """Gets the sub_path of this OrmVolumeV2.  # noqa: E501


        :return: The sub_path of this OrmVolumeV2.  # noqa: E501
        :rtype: str
        """
        return self._sub_path

    @sub_path.setter
    def sub_path(self, sub_path):
        """Sets the sub_path of this OrmVolumeV2.


        :param sub_path: The sub_path of this OrmVolumeV2.  # noqa: E501
        :type sub_path: str
        """

        self._sub_path = sub_path

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmVolumeV2.  # noqa: E501


        :return: The updated_dt of this OrmVolumeV2.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmVolumeV2.


        :param updated_dt: The updated_dt of this OrmVolumeV2.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmVolumeV2):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmVolumeV2):
            return True

        return self.to_dict() != other.to_dict()
