# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmLLMVectorDBConnection(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'auth_header_key': 'str',
        'authentication_type': 'str',
        'created_by_id': 'int',
        'created_dt': 'datetime',
        'db_type': 'str',
        'edges': 'OrmLLMVectorDBConnectionEdges',
        'encrypted_auth_header_value': 'str',
        'encrypted_bearer_token': 'str',
        'encrypted_password': 'str',
        'host': 'str',
        'id': 'int',
        'name': 'str',
        'organization_id': 'int',
        'port': 'str',
        'slug': 'str',
        'updated_dt': 'datetime',
        'username': 'str'
    }

    attribute_map = {
        'auth_header_key': 'auth_header_key',
        'authentication_type': 'authentication_type',
        'created_by_id': 'created_by_id',
        'created_dt': 'created_dt',
        'db_type': 'db_type',
        'edges': 'edges',
        'encrypted_auth_header_value': 'encrypted_auth_header_value',
        'encrypted_bearer_token': 'encrypted_bearer_token',
        'encrypted_password': 'encrypted_password',
        'host': 'host',
        'id': 'id',
        'name': 'name',
        'organization_id': 'organization_id',
        'port': 'port',
        'slug': 'slug',
        'updated_dt': 'updated_dt',
        'username': 'username'
    }

    def __init__(self, auth_header_key=None, authentication_type=None, created_by_id=None, created_dt=None, db_type=None, edges=None, encrypted_auth_header_value=None, encrypted_bearer_token=None, encrypted_password=None, host=None, id=None, name=None, organization_id=None, port=None, slug=None, updated_dt=None, username=None, local_vars_configuration=None):  # noqa: E501
        """OrmLLMVectorDBConnection - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._auth_header_key = None
        self._authentication_type = None
        self._created_by_id = None
        self._created_dt = None
        self._db_type = None
        self._edges = None
        self._encrypted_auth_header_value = None
        self._encrypted_bearer_token = None
        self._encrypted_password = None
        self._host = None
        self._id = None
        self._name = None
        self._organization_id = None
        self._port = None
        self._slug = None
        self._updated_dt = None
        self._username = None
        self.discriminator = None

        if auth_header_key is not None:
            self.auth_header_key = auth_header_key
        if authentication_type is not None:
            self.authentication_type = authentication_type
        if created_by_id is not None:
            self.created_by_id = created_by_id
        if created_dt is not None:
            self.created_dt = created_dt
        if db_type is not None:
            self.db_type = db_type
        if edges is not None:
            self.edges = edges
        if encrypted_auth_header_value is not None:
            self.encrypted_auth_header_value = encrypted_auth_header_value
        if encrypted_bearer_token is not None:
            self.encrypted_bearer_token = encrypted_bearer_token
        if encrypted_password is not None:
            self.encrypted_password = encrypted_password
        if host is not None:
            self.host = host
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if organization_id is not None:
            self.organization_id = organization_id
        if port is not None:
            self.port = port
        if slug is not None:
            self.slug = slug
        if updated_dt is not None:
            self.updated_dt = updated_dt
        if username is not None:
            self.username = username

    @property
    def auth_header_key(self):
        """Gets the auth_header_key of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The auth_header_key of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._auth_header_key

    @auth_header_key.setter
    def auth_header_key(self, auth_header_key):
        """Sets the auth_header_key of this OrmLLMVectorDBConnection.


        :param auth_header_key: The auth_header_key of this OrmLLMVectorDBConnection.  # noqa: E501
        :type auth_header_key: str
        """

        self._auth_header_key = auth_header_key

    @property
    def authentication_type(self):
        """Gets the authentication_type of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The authentication_type of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._authentication_type

    @authentication_type.setter
    def authentication_type(self, authentication_type):
        """Sets the authentication_type of this OrmLLMVectorDBConnection.


        :param authentication_type: The authentication_type of this OrmLLMVectorDBConnection.  # noqa: E501
        :type authentication_type: str
        """

        self._authentication_type = authentication_type

    @property
    def created_by_id(self):
        """Gets the created_by_id of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The created_by_id of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: int
        """
        return self._created_by_id

    @created_by_id.setter
    def created_by_id(self, created_by_id):
        """Sets the created_by_id of this OrmLLMVectorDBConnection.


        :param created_by_id: The created_by_id of this OrmLLMVectorDBConnection.  # noqa: E501
        :type created_by_id: int
        """

        self._created_by_id = created_by_id

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The created_dt of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmLLMVectorDBConnection.


        :param created_dt: The created_dt of this OrmLLMVectorDBConnection.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def db_type(self):
        """Gets the db_type of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The db_type of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._db_type

    @db_type.setter
    def db_type(self, db_type):
        """Sets the db_type of this OrmLLMVectorDBConnection.


        :param db_type: The db_type of this OrmLLMVectorDBConnection.  # noqa: E501
        :type db_type: str
        """

        self._db_type = db_type

    @property
    def edges(self):
        """Gets the edges of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The edges of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: OrmLLMVectorDBConnectionEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmLLMVectorDBConnection.


        :param edges: The edges of this OrmLLMVectorDBConnection.  # noqa: E501
        :type edges: OrmLLMVectorDBConnectionEdges
        """

        self._edges = edges

    @property
    def encrypted_auth_header_value(self):
        """Gets the encrypted_auth_header_value of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The encrypted_auth_header_value of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._encrypted_auth_header_value

    @encrypted_auth_header_value.setter
    def encrypted_auth_header_value(self, encrypted_auth_header_value):
        """Sets the encrypted_auth_header_value of this OrmLLMVectorDBConnection.


        :param encrypted_auth_header_value: The encrypted_auth_header_value of this OrmLLMVectorDBConnection.  # noqa: E501
        :type encrypted_auth_header_value: str
        """

        self._encrypted_auth_header_value = encrypted_auth_header_value

    @property
    def encrypted_bearer_token(self):
        """Gets the encrypted_bearer_token of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The encrypted_bearer_token of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._encrypted_bearer_token

    @encrypted_bearer_token.setter
    def encrypted_bearer_token(self, encrypted_bearer_token):
        """Sets the encrypted_bearer_token of this OrmLLMVectorDBConnection.


        :param encrypted_bearer_token: The encrypted_bearer_token of this OrmLLMVectorDBConnection.  # noqa: E501
        :type encrypted_bearer_token: str
        """

        self._encrypted_bearer_token = encrypted_bearer_token

    @property
    def encrypted_password(self):
        """Gets the encrypted_password of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The encrypted_password of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._encrypted_password

    @encrypted_password.setter
    def encrypted_password(self, encrypted_password):
        """Sets the encrypted_password of this OrmLLMVectorDBConnection.


        :param encrypted_password: The encrypted_password of this OrmLLMVectorDBConnection.  # noqa: E501
        :type encrypted_password: str
        """

        self._encrypted_password = encrypted_password

    @property
    def host(self):
        """Gets the host of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The host of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._host

    @host.setter
    def host(self, host):
        """Sets the host of this OrmLLMVectorDBConnection.


        :param host: The host of this OrmLLMVectorDBConnection.  # noqa: E501
        :type host: str
        """

        self._host = host

    @property
    def id(self):
        """Gets the id of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The id of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmLLMVectorDBConnection.


        :param id: The id of this OrmLLMVectorDBConnection.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The name of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrmLLMVectorDBConnection.


        :param name: The name of this OrmLLMVectorDBConnection.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def organization_id(self):
        """Gets the organization_id of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The organization_id of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrmLLMVectorDBConnection.


        :param organization_id: The organization_id of this OrmLLMVectorDBConnection.  # noqa: E501
        :type organization_id: int
        """

        self._organization_id = organization_id

    @property
    def port(self):
        """Gets the port of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The port of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._port

    @port.setter
    def port(self, port):
        """Sets the port of this OrmLLMVectorDBConnection.


        :param port: The port of this OrmLLMVectorDBConnection.  # noqa: E501
        :type port: str
        """

        self._port = port

    @property
    def slug(self):
        """Gets the slug of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The slug of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._slug

    @slug.setter
    def slug(self, slug):
        """Sets the slug of this OrmLLMVectorDBConnection.


        :param slug: The slug of this OrmLLMVectorDBConnection.  # noqa: E501
        :type slug: str
        """

        self._slug = slug

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The updated_dt of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmLLMVectorDBConnection.


        :param updated_dt: The updated_dt of this OrmLLMVectorDBConnection.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def username(self):
        """Gets the username of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The username of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """Sets the username of this OrmLLMVectorDBConnection.


        :param username: The username of this OrmLLMVectorDBConnection.  # noqa: E501
        :type username: str
        """

        self._username = username

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmLLMVectorDBConnection):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmLLMVectorDBConnection):
            return True

        return self.to_dict() != other.to_dict()
