# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmKernelImageEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'model_service_revisions': 'list[OrmModelServiceRevision]',
        'organization': 'OrmOrganization',
        'organization_credentials': 'OrmOrganizationCredentials',
        'run_specs': 'list[OrmRunSpec]'
    }

    attribute_map = {
        'model_service_revisions': 'model_service_revisions',
        'organization': 'organization',
        'organization_credentials': 'organization_credentials',
        'run_specs': 'run_specs'
    }

    def __init__(self, model_service_revisions=None, organization=None, organization_credentials=None, run_specs=None, local_vars_configuration=None):  # noqa: E501
        """OrmKernelImageEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._model_service_revisions = None
        self._organization = None
        self._organization_credentials = None
        self._run_specs = None
        self.discriminator = None

        if model_service_revisions is not None:
            self.model_service_revisions = model_service_revisions
        if organization is not None:
            self.organization = organization
        if organization_credentials is not None:
            self.organization_credentials = organization_credentials
        if run_specs is not None:
            self.run_specs = run_specs

    @property
    def model_service_revisions(self):
        """Gets the model_service_revisions of this OrmKernelImageEdges.  # noqa: E501


        :return: The model_service_revisions of this OrmKernelImageEdges.  # noqa: E501
        :rtype: list[OrmModelServiceRevision]
        """
        return self._model_service_revisions

    @model_service_revisions.setter
    def model_service_revisions(self, model_service_revisions):
        """Sets the model_service_revisions of this OrmKernelImageEdges.


        :param model_service_revisions: The model_service_revisions of this OrmKernelImageEdges.  # noqa: E501
        :type model_service_revisions: list[OrmModelServiceRevision]
        """

        self._model_service_revisions = model_service_revisions

    @property
    def organization(self):
        """Gets the organization of this OrmKernelImageEdges.  # noqa: E501


        :return: The organization of this OrmKernelImageEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrmKernelImageEdges.


        :param organization: The organization of this OrmKernelImageEdges.  # noqa: E501
        :type organization: OrmOrganization
        """

        self._organization = organization

    @property
    def organization_credentials(self):
        """Gets the organization_credentials of this OrmKernelImageEdges.  # noqa: E501


        :return: The organization_credentials of this OrmKernelImageEdges.  # noqa: E501
        :rtype: OrmOrganizationCredentials
        """
        return self._organization_credentials

    @organization_credentials.setter
    def organization_credentials(self, organization_credentials):
        """Sets the organization_credentials of this OrmKernelImageEdges.


        :param organization_credentials: The organization_credentials of this OrmKernelImageEdges.  # noqa: E501
        :type organization_credentials: OrmOrganizationCredentials
        """

        self._organization_credentials = organization_credentials

    @property
    def run_specs(self):
        """Gets the run_specs of this OrmKernelImageEdges.  # noqa: E501


        :return: The run_specs of this OrmKernelImageEdges.  # noqa: E501
        :rtype: list[OrmRunSpec]
        """
        return self._run_specs

    @run_specs.setter
    def run_specs(self, run_specs):
        """Sets the run_specs of this OrmKernelImageEdges.


        :param run_specs: The run_specs of this OrmKernelImageEdges.  # noqa: E501
        :type run_specs: list[OrmRunSpec]
        """

        self._run_specs = run_specs

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmKernelImageEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmKernelImageEdges):
            return True

        return self.to_dict() != other.to_dict()
