# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmRunExecution(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'arguments': 'dict[str, object]',
        'created_dt': 'datetime',
        'edges': 'OrmRunExecutionEdges',
        'execution_created_by': 'int',
        'exposed_services': 'dict[str, object]',
        'final_kernel_resource_spec': 'dict[str, object]',
        'id': 'int',
        'immutable_slug': 'str',
        'interactive_config': 'dict[str, object]',
        'interactive_end_dt': 'datetime',
        'local_experiment_id': 'int',
        'message': 'str',
        'metrics_summary': 'dict[str, object]',
        'organization_id': 'int',
        'pipeline_step_execution_id': 'int',
        'progress_percent': 'int',
        'project_id': 'int',
        'run_spec_id': 'int',
        'star': 'bool',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'arguments': 'arguments',
        'created_dt': 'created_dt',
        'edges': 'edges',
        'execution_created_by': 'execution_created_by',
        'exposed_services': 'exposed_services',
        'final_kernel_resource_spec': 'final_kernel_resource_spec',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'interactive_config': 'interactive_config',
        'interactive_end_dt': 'interactive_end_dt',
        'local_experiment_id': 'local_experiment_id',
        'message': 'message',
        'metrics_summary': 'metrics_summary',
        'organization_id': 'organization_id',
        'pipeline_step_execution_id': 'pipeline_step_execution_id',
        'progress_percent': 'progress_percent',
        'project_id': 'project_id',
        'run_spec_id': 'run_spec_id',
        'star': 'star',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, arguments=None, created_dt=None, edges=None, execution_created_by=None, exposed_services=None, final_kernel_resource_spec=None, id=None, immutable_slug=None, interactive_config=None, interactive_end_dt=None, local_experiment_id=None, message=None, metrics_summary=None, organization_id=None, pipeline_step_execution_id=None, progress_percent=None, project_id=None, run_spec_id=None, star=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """OrmRunExecution - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._arguments = None
        self._created_dt = None
        self._edges = None
        self._execution_created_by = None
        self._exposed_services = None
        self._final_kernel_resource_spec = None
        self._id = None
        self._immutable_slug = None
        self._interactive_config = None
        self._interactive_end_dt = None
        self._local_experiment_id = None
        self._message = None
        self._metrics_summary = None
        self._organization_id = None
        self._pipeline_step_execution_id = None
        self._progress_percent = None
        self._project_id = None
        self._run_spec_id = None
        self._star = None
        self._updated_dt = None
        self.discriminator = None

        if arguments is not None:
            self.arguments = arguments
        if created_dt is not None:
            self.created_dt = created_dt
        if edges is not None:
            self.edges = edges
        self.execution_created_by = execution_created_by
        if exposed_services is not None:
            self.exposed_services = exposed_services
        if final_kernel_resource_spec is not None:
            self.final_kernel_resource_spec = final_kernel_resource_spec
        if id is not None:
            self.id = id
        if immutable_slug is not None:
            self.immutable_slug = immutable_slug
        if interactive_config is not None:
            self.interactive_config = interactive_config
        self.interactive_end_dt = interactive_end_dt
        self.local_experiment_id = local_experiment_id
        if message is not None:
            self.message = message
        if metrics_summary is not None:
            self.metrics_summary = metrics_summary
        if organization_id is not None:
            self.organization_id = organization_id
        self.pipeline_step_execution_id = pipeline_step_execution_id
        self.progress_percent = progress_percent
        self.project_id = project_id
        if run_spec_id is not None:
            self.run_spec_id = run_spec_id
        if star is not None:
            self.star = star
        if updated_dt is not None:
            self.updated_dt = updated_dt

    @property
    def arguments(self):
        """Gets the arguments of this OrmRunExecution.  # noqa: E501


        :return: The arguments of this OrmRunExecution.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._arguments

    @arguments.setter
    def arguments(self, arguments):
        """Sets the arguments of this OrmRunExecution.


        :param arguments: The arguments of this OrmRunExecution.  # noqa: E501
        :type arguments: dict[str, object]
        """

        self._arguments = arguments

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmRunExecution.  # noqa: E501


        :return: The created_dt of this OrmRunExecution.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmRunExecution.


        :param created_dt: The created_dt of this OrmRunExecution.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def edges(self):
        """Gets the edges of this OrmRunExecution.  # noqa: E501


        :return: The edges of this OrmRunExecution.  # noqa: E501
        :rtype: OrmRunExecutionEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmRunExecution.


        :param edges: The edges of this OrmRunExecution.  # noqa: E501
        :type edges: OrmRunExecutionEdges
        """

        self._edges = edges

    @property
    def execution_created_by(self):
        """Gets the execution_created_by of this OrmRunExecution.  # noqa: E501


        :return: The execution_created_by of this OrmRunExecution.  # noqa: E501
        :rtype: int
        """
        return self._execution_created_by

    @execution_created_by.setter
    def execution_created_by(self, execution_created_by):
        """Sets the execution_created_by of this OrmRunExecution.


        :param execution_created_by: The execution_created_by of this OrmRunExecution.  # noqa: E501
        :type execution_created_by: int
        """

        self._execution_created_by = execution_created_by

    @property
    def exposed_services(self):
        """Gets the exposed_services of this OrmRunExecution.  # noqa: E501


        :return: The exposed_services of this OrmRunExecution.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._exposed_services

    @exposed_services.setter
    def exposed_services(self, exposed_services):
        """Sets the exposed_services of this OrmRunExecution.


        :param exposed_services: The exposed_services of this OrmRunExecution.  # noqa: E501
        :type exposed_services: dict[str, object]
        """

        self._exposed_services = exposed_services

    @property
    def final_kernel_resource_spec(self):
        """Gets the final_kernel_resource_spec of this OrmRunExecution.  # noqa: E501


        :return: The final_kernel_resource_spec of this OrmRunExecution.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._final_kernel_resource_spec

    @final_kernel_resource_spec.setter
    def final_kernel_resource_spec(self, final_kernel_resource_spec):
        """Sets the final_kernel_resource_spec of this OrmRunExecution.


        :param final_kernel_resource_spec: The final_kernel_resource_spec of this OrmRunExecution.  # noqa: E501
        :type final_kernel_resource_spec: dict[str, object]
        """

        self._final_kernel_resource_spec = final_kernel_resource_spec

    @property
    def id(self):
        """Gets the id of this OrmRunExecution.  # noqa: E501


        :return: The id of this OrmRunExecution.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmRunExecution.


        :param id: The id of this OrmRunExecution.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this OrmRunExecution.  # noqa: E501


        :return: The immutable_slug of this OrmRunExecution.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this OrmRunExecution.


        :param immutable_slug: The immutable_slug of this OrmRunExecution.  # noqa: E501
        :type immutable_slug: str
        """

        self._immutable_slug = immutable_slug

    @property
    def interactive_config(self):
        """Gets the interactive_config of this OrmRunExecution.  # noqa: E501


        :return: The interactive_config of this OrmRunExecution.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._interactive_config

    @interactive_config.setter
    def interactive_config(self, interactive_config):
        """Sets the interactive_config of this OrmRunExecution.


        :param interactive_config: The interactive_config of this OrmRunExecution.  # noqa: E501
        :type interactive_config: dict[str, object]
        """

        self._interactive_config = interactive_config

    @property
    def interactive_end_dt(self):
        """Gets the interactive_end_dt of this OrmRunExecution.  # noqa: E501


        :return: The interactive_end_dt of this OrmRunExecution.  # noqa: E501
        :rtype: datetime
        """
        return self._interactive_end_dt

    @interactive_end_dt.setter
    def interactive_end_dt(self, interactive_end_dt):
        """Sets the interactive_end_dt of this OrmRunExecution.


        :param interactive_end_dt: The interactive_end_dt of this OrmRunExecution.  # noqa: E501
        :type interactive_end_dt: datetime
        """

        self._interactive_end_dt = interactive_end_dt

    @property
    def local_experiment_id(self):
        """Gets the local_experiment_id of this OrmRunExecution.  # noqa: E501


        :return: The local_experiment_id of this OrmRunExecution.  # noqa: E501
        :rtype: int
        """
        return self._local_experiment_id

    @local_experiment_id.setter
    def local_experiment_id(self, local_experiment_id):
        """Sets the local_experiment_id of this OrmRunExecution.


        :param local_experiment_id: The local_experiment_id of this OrmRunExecution.  # noqa: E501
        :type local_experiment_id: int
        """

        self._local_experiment_id = local_experiment_id

    @property
    def message(self):
        """Gets the message of this OrmRunExecution.  # noqa: E501


        :return: The message of this OrmRunExecution.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this OrmRunExecution.


        :param message: The message of this OrmRunExecution.  # noqa: E501
        :type message: str
        """

        self._message = message

    @property
    def metrics_summary(self):
        """Gets the metrics_summary of this OrmRunExecution.  # noqa: E501


        :return: The metrics_summary of this OrmRunExecution.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._metrics_summary

    @metrics_summary.setter
    def metrics_summary(self, metrics_summary):
        """Sets the metrics_summary of this OrmRunExecution.


        :param metrics_summary: The metrics_summary of this OrmRunExecution.  # noqa: E501
        :type metrics_summary: dict[str, object]
        """

        self._metrics_summary = metrics_summary

    @property
    def organization_id(self):
        """Gets the organization_id of this OrmRunExecution.  # noqa: E501


        :return: The organization_id of this OrmRunExecution.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrmRunExecution.


        :param organization_id: The organization_id of this OrmRunExecution.  # noqa: E501
        :type organization_id: int
        """

        self._organization_id = organization_id

    @property
    def pipeline_step_execution_id(self):
        """Gets the pipeline_step_execution_id of this OrmRunExecution.  # noqa: E501


        :return: The pipeline_step_execution_id of this OrmRunExecution.  # noqa: E501
        :rtype: int
        """
        return self._pipeline_step_execution_id

    @pipeline_step_execution_id.setter
    def pipeline_step_execution_id(self, pipeline_step_execution_id):
        """Sets the pipeline_step_execution_id of this OrmRunExecution.


        :param pipeline_step_execution_id: The pipeline_step_execution_id of this OrmRunExecution.  # noqa: E501
        :type pipeline_step_execution_id: int
        """

        self._pipeline_step_execution_id = pipeline_step_execution_id

    @property
    def progress_percent(self):
        """Gets the progress_percent of this OrmRunExecution.  # noqa: E501


        :return: The progress_percent of this OrmRunExecution.  # noqa: E501
        :rtype: int
        """
        return self._progress_percent

    @progress_percent.setter
    def progress_percent(self, progress_percent):
        """Sets the progress_percent of this OrmRunExecution.


        :param progress_percent: The progress_percent of this OrmRunExecution.  # noqa: E501
        :type progress_percent: int
        """

        self._progress_percent = progress_percent

    @property
    def project_id(self):
        """Gets the project_id of this OrmRunExecution.  # noqa: E501


        :return: The project_id of this OrmRunExecution.  # noqa: E501
        :rtype: int
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this OrmRunExecution.


        :param project_id: The project_id of this OrmRunExecution.  # noqa: E501
        :type project_id: int
        """

        self._project_id = project_id

    @property
    def run_spec_id(self):
        """Gets the run_spec_id of this OrmRunExecution.  # noqa: E501


        :return: The run_spec_id of this OrmRunExecution.  # noqa: E501
        :rtype: int
        """
        return self._run_spec_id

    @run_spec_id.setter
    def run_spec_id(self, run_spec_id):
        """Sets the run_spec_id of this OrmRunExecution.


        :param run_spec_id: The run_spec_id of this OrmRunExecution.  # noqa: E501
        :type run_spec_id: int
        """

        self._run_spec_id = run_spec_id

    @property
    def star(self):
        """Gets the star of this OrmRunExecution.  # noqa: E501


        :return: The star of this OrmRunExecution.  # noqa: E501
        :rtype: bool
        """
        return self._star

    @star.setter
    def star(self, star):
        """Sets the star of this OrmRunExecution.


        :param star: The star of this OrmRunExecution.  # noqa: E501
        :type star: bool
        """

        self._star = star

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmRunExecution.  # noqa: E501


        :return: The updated_dt of this OrmRunExecution.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmRunExecution.


        :param updated_dt: The updated_dt of this OrmRunExecution.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmRunExecution):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmRunExecution):
            return True

        return self.to_dict() != other.to_dict()
