# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmWalletHistory(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'additional_data': 'dict[str, object]',
        'artifact_id': 'int',
        'balance_from': 'int',
        'balance_to': 'int',
        'cluster_id': 'int',
        'coupon_redemption_id': 'int',
        'created_dt': 'datetime',
        'edges': 'OrmWalletHistoryEdges',
        'id': 'int',
        'organization_id': 'int',
        'reason': 'str',
        'type': 'str',
        'updated_dt': 'datetime',
        'wallet_id': 'int',
        'workload_id': 'int'
    }

    attribute_map = {
        'additional_data': 'additional_data',
        'artifact_id': 'artifact_id',
        'balance_from': 'balance_from',
        'balance_to': 'balance_to',
        'cluster_id': 'cluster_id',
        'coupon_redemption_id': 'coupon_redemption_id',
        'created_dt': 'created_dt',
        'edges': 'edges',
        'id': 'id',
        'organization_id': 'organization_id',
        'reason': 'reason',
        'type': 'type',
        'updated_dt': 'updated_dt',
        'wallet_id': 'wallet_id',
        'workload_id': 'workload_id'
    }

    def __init__(self, additional_data=None, artifact_id=None, balance_from=None, balance_to=None, cluster_id=None, coupon_redemption_id=None, created_dt=None, edges=None, id=None, organization_id=None, reason=None, type=None, updated_dt=None, wallet_id=None, workload_id=None, local_vars_configuration=None):  # noqa: E501
        """OrmWalletHistory - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._additional_data = None
        self._artifact_id = None
        self._balance_from = None
        self._balance_to = None
        self._cluster_id = None
        self._coupon_redemption_id = None
        self._created_dt = None
        self._edges = None
        self._id = None
        self._organization_id = None
        self._reason = None
        self._type = None
        self._updated_dt = None
        self._wallet_id = None
        self._workload_id = None
        self.discriminator = None

        if additional_data is not None:
            self.additional_data = additional_data
        self.artifact_id = artifact_id
        if balance_from is not None:
            self.balance_from = balance_from
        if balance_to is not None:
            self.balance_to = balance_to
        self.cluster_id = cluster_id
        self.coupon_redemption_id = coupon_redemption_id
        if created_dt is not None:
            self.created_dt = created_dt
        if edges is not None:
            self.edges = edges
        if id is not None:
            self.id = id
        self.organization_id = organization_id
        if reason is not None:
            self.reason = reason
        if type is not None:
            self.type = type
        if updated_dt is not None:
            self.updated_dt = updated_dt
        if wallet_id is not None:
            self.wallet_id = wallet_id
        self.workload_id = workload_id

    @property
    def additional_data(self):
        """Gets the additional_data of this OrmWalletHistory.  # noqa: E501


        :return: The additional_data of this OrmWalletHistory.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._additional_data

    @additional_data.setter
    def additional_data(self, additional_data):
        """Sets the additional_data of this OrmWalletHistory.


        :param additional_data: The additional_data of this OrmWalletHistory.  # noqa: E501
        :type additional_data: dict[str, object]
        """

        self._additional_data = additional_data

    @property
    def artifact_id(self):
        """Gets the artifact_id of this OrmWalletHistory.  # noqa: E501


        :return: The artifact_id of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._artifact_id

    @artifact_id.setter
    def artifact_id(self, artifact_id):
        """Sets the artifact_id of this OrmWalletHistory.


        :param artifact_id: The artifact_id of this OrmWalletHistory.  # noqa: E501
        :type artifact_id: int
        """

        self._artifact_id = artifact_id

    @property
    def balance_from(self):
        """Gets the balance_from of this OrmWalletHistory.  # noqa: E501


        :return: The balance_from of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._balance_from

    @balance_from.setter
    def balance_from(self, balance_from):
        """Sets the balance_from of this OrmWalletHistory.


        :param balance_from: The balance_from of this OrmWalletHistory.  # noqa: E501
        :type balance_from: int
        """

        self._balance_from = balance_from

    @property
    def balance_to(self):
        """Gets the balance_to of this OrmWalletHistory.  # noqa: E501


        :return: The balance_to of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._balance_to

    @balance_to.setter
    def balance_to(self, balance_to):
        """Sets the balance_to of this OrmWalletHistory.


        :param balance_to: The balance_to of this OrmWalletHistory.  # noqa: E501
        :type balance_to: int
        """

        self._balance_to = balance_to

    @property
    def cluster_id(self):
        """Gets the cluster_id of this OrmWalletHistory.  # noqa: E501


        :return: The cluster_id of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this OrmWalletHistory.


        :param cluster_id: The cluster_id of this OrmWalletHistory.  # noqa: E501
        :type cluster_id: int
        """

        self._cluster_id = cluster_id

    @property
    def coupon_redemption_id(self):
        """Gets the coupon_redemption_id of this OrmWalletHistory.  # noqa: E501


        :return: The coupon_redemption_id of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._coupon_redemption_id

    @coupon_redemption_id.setter
    def coupon_redemption_id(self, coupon_redemption_id):
        """Sets the coupon_redemption_id of this OrmWalletHistory.


        :param coupon_redemption_id: The coupon_redemption_id of this OrmWalletHistory.  # noqa: E501
        :type coupon_redemption_id: int
        """

        self._coupon_redemption_id = coupon_redemption_id

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmWalletHistory.  # noqa: E501


        :return: The created_dt of this OrmWalletHistory.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmWalletHistory.


        :param created_dt: The created_dt of this OrmWalletHistory.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def edges(self):
        """Gets the edges of this OrmWalletHistory.  # noqa: E501


        :return: The edges of this OrmWalletHistory.  # noqa: E501
        :rtype: OrmWalletHistoryEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmWalletHistory.


        :param edges: The edges of this OrmWalletHistory.  # noqa: E501
        :type edges: OrmWalletHistoryEdges
        """

        self._edges = edges

    @property
    def id(self):
        """Gets the id of this OrmWalletHistory.  # noqa: E501


        :return: The id of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmWalletHistory.


        :param id: The id of this OrmWalletHistory.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def organization_id(self):
        """Gets the organization_id of this OrmWalletHistory.  # noqa: E501


        :return: The organization_id of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrmWalletHistory.


        :param organization_id: The organization_id of this OrmWalletHistory.  # noqa: E501
        :type organization_id: int
        """

        self._organization_id = organization_id

    @property
    def reason(self):
        """Gets the reason of this OrmWalletHistory.  # noqa: E501


        :return: The reason of this OrmWalletHistory.  # noqa: E501
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Sets the reason of this OrmWalletHistory.


        :param reason: The reason of this OrmWalletHistory.  # noqa: E501
        :type reason: str
        """

        self._reason = reason

    @property
    def type(self):
        """Gets the type of this OrmWalletHistory.  # noqa: E501


        :return: The type of this OrmWalletHistory.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this OrmWalletHistory.


        :param type: The type of this OrmWalletHistory.  # noqa: E501
        :type type: str
        """

        self._type = type

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmWalletHistory.  # noqa: E501


        :return: The updated_dt of this OrmWalletHistory.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmWalletHistory.


        :param updated_dt: The updated_dt of this OrmWalletHistory.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def wallet_id(self):
        """Gets the wallet_id of this OrmWalletHistory.  # noqa: E501


        :return: The wallet_id of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._wallet_id

    @wallet_id.setter
    def wallet_id(self, wallet_id):
        """Sets the wallet_id of this OrmWalletHistory.


        :param wallet_id: The wallet_id of this OrmWalletHistory.  # noqa: E501
        :type wallet_id: int
        """

        self._wallet_id = wallet_id

    @property
    def workload_id(self):
        """Gets the workload_id of this OrmWalletHistory.  # noqa: E501


        :return: The workload_id of this OrmWalletHistory.  # noqa: E501
        :rtype: int
        """
        return self._workload_id

    @workload_id.setter
    def workload_id(self, workload_id):
        """Sets the workload_id of this OrmWalletHistory.


        :param workload_id: The workload_id of this OrmWalletHistory.  # noqa: E501
        :type workload_id: int
        """

        self._workload_id = workload_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmWalletHistory):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmWalletHistory):
            return True

        return self.to_dict() != other.to_dict()
