# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class HubModelTaskCreateAPIInput(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'category_id': 'int',
        'description': 'str',
        'key': 'str',
        'order_number': 'int',
        'overview': 'str',
        'scope': 'str',
        'status': 'str',
        'tags': 'list[ProtoTag]',
        'thumbnail_url': 'str',
        'title': 'str',
        'type': 'str',
        'yaml': 'str'
    }

    attribute_map = {
        'category_id': 'category_id',
        'description': 'description',
        'key': 'key',
        'order_number': 'order_number',
        'overview': 'overview',
        'scope': 'scope',
        'status': 'status',
        'tags': 'tags',
        'thumbnail_url': 'thumbnail_url',
        'title': 'title',
        'type': 'type',
        'yaml': 'yaml'
    }

    def __init__(self, category_id=None, description=None, key=None, order_number=None, overview=None, scope='private', status=None, tags=None, thumbnail_url=None, title=None, type=None, yaml=None, local_vars_configuration=None):  # noqa: E501
        """HubModelTaskCreateAPIInput - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._category_id = None
        self._description = None
        self._key = None
        self._order_number = None
        self._overview = None
        self._scope = None
        self._status = None
        self._tags = None
        self._thumbnail_url = None
        self._title = None
        self._type = None
        self._yaml = None
        self.discriminator = None

        self.category_id = category_id
        self.description = description
        self.key = key
        self.order_number = order_number
        self.overview = overview
        if scope is not None:
            self.scope = scope
        self.status = status
        if tags is not None:
            self.tags = tags
        self.thumbnail_url = thumbnail_url
        self.title = title
        self.type = type
        if yaml is not None:
            self.yaml = yaml

    @property
    def category_id(self):
        """Gets the category_id of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The category_id of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._category_id

    @category_id.setter
    def category_id(self, category_id):
        """Sets the category_id of this HubModelTaskCreateAPIInput.


        :param category_id: The category_id of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type category_id: int
        """

        self._category_id = category_id

    @property
    def description(self):
        """Gets the description of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The description of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this HubModelTaskCreateAPIInput.


        :param description: The description of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def key(self):
        """Gets the key of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The key of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this HubModelTaskCreateAPIInput.


        :param key: The key of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type key: str
        """

        self._key = key

    @property
    def order_number(self):
        """Gets the order_number of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The order_number of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._order_number

    @order_number.setter
    def order_number(self, order_number):
        """Sets the order_number of this HubModelTaskCreateAPIInput.


        :param order_number: The order_number of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type order_number: int
        """

        self._order_number = order_number

    @property
    def overview(self):
        """Gets the overview of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The overview of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._overview

    @overview.setter
    def overview(self, overview):
        """Sets the overview of this HubModelTaskCreateAPIInput.


        :param overview: The overview of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type overview: str
        """

        self._overview = overview

    @property
    def scope(self):
        """Gets the scope of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The scope of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._scope

    @scope.setter
    def scope(self, scope):
        """Sets the scope of this HubModelTaskCreateAPIInput.


        :param scope: The scope of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type scope: str
        """
        allowed_values = ["private", "organization", "public"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and scope not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `scope` ({0}), must be one of {1}"  # noqa: E501
                .format(scope, allowed_values)
            )

        self._scope = scope

    @property
    def status(self):
        """Gets the status of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The status of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this HubModelTaskCreateAPIInput.


        :param status: The status of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501
        allowed_values = ["draft", "published"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and status not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}"  # noqa: E501
                .format(status, allowed_values)
            )

        self._status = status

    @property
    def tags(self):
        """Gets the tags of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The tags of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: list[ProtoTag]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this HubModelTaskCreateAPIInput.


        :param tags: The tags of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type tags: list[ProtoTag]
        """

        self._tags = tags

    @property
    def thumbnail_url(self):
        """Gets the thumbnail_url of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The thumbnail_url of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._thumbnail_url

    @thumbnail_url.setter
    def thumbnail_url(self, thumbnail_url):
        """Sets the thumbnail_url of this HubModelTaskCreateAPIInput.


        :param thumbnail_url: The thumbnail_url of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type thumbnail_url: str
        """

        self._thumbnail_url = thumbnail_url

    @property
    def title(self):
        """Gets the title of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The title of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this HubModelTaskCreateAPIInput.


        :param title: The title of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type title: str
        """
        if self.local_vars_configuration.client_side_validation and title is None:  # noqa: E501
            raise ValueError("Invalid value for `title`, must not be `None`")  # noqa: E501

        self._title = title

    @property
    def type(self):
        """Gets the type of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The type of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this HubModelTaskCreateAPIInput.


        :param type: The type of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["run", "service", "serverless", "pipeline"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def yaml(self):
        """Gets the yaml of this HubModelTaskCreateAPIInput.  # noqa: E501


        :return: The yaml of this HubModelTaskCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._yaml

    @yaml.setter
    def yaml(self, yaml):
        """Sets the yaml of this HubModelTaskCreateAPIInput.


        :param yaml: The yaml of this HubModelTaskCreateAPIInput.  # noqa: E501
        :type yaml: str
        """

        self._yaml = yaml

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, HubModelTaskCreateAPIInput):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, HubModelTaskCreateAPIInput):
            return True

        return self.to_dict() != other.to_dict()
