# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmLLMModelConnection(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'aws_region': 'str',
        'created_by_id': 'int',
        'created_dt': 'datetime',
        'description': 'str',
        'edges': 'OrmLLMModelConnectionEdges',
        'id': 'int',
        'name': 'str',
        'organization_id': 'int',
        'provider': 'str',
        'secret_id': 'int',
        'slug': 'str',
        'type': 'str',
        'updated_dt': 'datetime',
        'url': 'str'
    }

    attribute_map = {
        'aws_region': 'aws_region',
        'created_by_id': 'created_by_id',
        'created_dt': 'created_dt',
        'description': 'description',
        'edges': 'edges',
        'id': 'id',
        'name': 'name',
        'organization_id': 'organization_id',
        'provider': 'provider',
        'secret_id': 'secret_id',
        'slug': 'slug',
        'type': 'type',
        'updated_dt': 'updated_dt',
        'url': 'url'
    }

    def __init__(self, aws_region=None, created_by_id=None, created_dt=None, description=None, edges=None, id=None, name=None, organization_id=None, provider=None, secret_id=None, slug=None, type=None, updated_dt=None, url=None, local_vars_configuration=None):  # noqa: E501
        """OrmLLMModelConnection - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._aws_region = None
        self._created_by_id = None
        self._created_dt = None
        self._description = None
        self._edges = None
        self._id = None
        self._name = None
        self._organization_id = None
        self._provider = None
        self._secret_id = None
        self._slug = None
        self._type = None
        self._updated_dt = None
        self._url = None
        self.discriminator = None

        self.aws_region = aws_region
        if created_by_id is not None:
            self.created_by_id = created_by_id
        if created_dt is not None:
            self.created_dt = created_dt
        if description is not None:
            self.description = description
        if edges is not None:
            self.edges = edges
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if organization_id is not None:
            self.organization_id = organization_id
        if provider is not None:
            self.provider = provider
        self.secret_id = secret_id
        if slug is not None:
            self.slug = slug
        if type is not None:
            self.type = type
        if updated_dt is not None:
            self.updated_dt = updated_dt
        self.url = url

    @property
    def aws_region(self):
        """Gets the aws_region of this OrmLLMModelConnection.  # noqa: E501


        :return: The aws_region of this OrmLLMModelConnection.  # noqa: E501
        :rtype: str
        """
        return self._aws_region

    @aws_region.setter
    def aws_region(self, aws_region):
        """Sets the aws_region of this OrmLLMModelConnection.


        :param aws_region: The aws_region of this OrmLLMModelConnection.  # noqa: E501
        :type aws_region: str
        """

        self._aws_region = aws_region

    @property
    def created_by_id(self):
        """Gets the created_by_id of this OrmLLMModelConnection.  # noqa: E501


        :return: The created_by_id of this OrmLLMModelConnection.  # noqa: E501
        :rtype: int
        """
        return self._created_by_id

    @created_by_id.setter
    def created_by_id(self, created_by_id):
        """Sets the created_by_id of this OrmLLMModelConnection.


        :param created_by_id: The created_by_id of this OrmLLMModelConnection.  # noqa: E501
        :type created_by_id: int
        """

        self._created_by_id = created_by_id

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmLLMModelConnection.  # noqa: E501


        :return: The created_dt of this OrmLLMModelConnection.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmLLMModelConnection.


        :param created_dt: The created_dt of this OrmLLMModelConnection.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def description(self):
        """Gets the description of this OrmLLMModelConnection.  # noqa: E501


        :return: The description of this OrmLLMModelConnection.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this OrmLLMModelConnection.


        :param description: The description of this OrmLLMModelConnection.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def edges(self):
        """Gets the edges of this OrmLLMModelConnection.  # noqa: E501


        :return: The edges of this OrmLLMModelConnection.  # noqa: E501
        :rtype: OrmLLMModelConnectionEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmLLMModelConnection.


        :param edges: The edges of this OrmLLMModelConnection.  # noqa: E501
        :type edges: OrmLLMModelConnectionEdges
        """

        self._edges = edges

    @property
    def id(self):
        """Gets the id of this OrmLLMModelConnection.  # noqa: E501


        :return: The id of this OrmLLMModelConnection.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmLLMModelConnection.


        :param id: The id of this OrmLLMModelConnection.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this OrmLLMModelConnection.  # noqa: E501


        :return: The name of this OrmLLMModelConnection.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrmLLMModelConnection.


        :param name: The name of this OrmLLMModelConnection.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def organization_id(self):
        """Gets the organization_id of this OrmLLMModelConnection.  # noqa: E501


        :return: The organization_id of this OrmLLMModelConnection.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrmLLMModelConnection.


        :param organization_id: The organization_id of this OrmLLMModelConnection.  # noqa: E501
        :type organization_id: int
        """

        self._organization_id = organization_id

    @property
    def provider(self):
        """Gets the provider of this OrmLLMModelConnection.  # noqa: E501


        :return: The provider of this OrmLLMModelConnection.  # noqa: E501
        :rtype: str
        """
        return self._provider

    @provider.setter
    def provider(self, provider):
        """Sets the provider of this OrmLLMModelConnection.


        :param provider: The provider of this OrmLLMModelConnection.  # noqa: E501
        :type provider: str
        """

        self._provider = provider

    @property
    def secret_id(self):
        """Gets the secret_id of this OrmLLMModelConnection.  # noqa: E501


        :return: The secret_id of this OrmLLMModelConnection.  # noqa: E501
        :rtype: int
        """
        return self._secret_id

    @secret_id.setter
    def secret_id(self, secret_id):
        """Sets the secret_id of this OrmLLMModelConnection.


        :param secret_id: The secret_id of this OrmLLMModelConnection.  # noqa: E501
        :type secret_id: int
        """

        self._secret_id = secret_id

    @property
    def slug(self):
        """Gets the slug of this OrmLLMModelConnection.  # noqa: E501


        :return: The slug of this OrmLLMModelConnection.  # noqa: E501
        :rtype: str
        """
        return self._slug

    @slug.setter
    def slug(self, slug):
        """Sets the slug of this OrmLLMModelConnection.


        :param slug: The slug of this OrmLLMModelConnection.  # noqa: E501
        :type slug: str
        """

        self._slug = slug

    @property
    def type(self):
        """Gets the type of this OrmLLMModelConnection.  # noqa: E501


        :return: The type of this OrmLLMModelConnection.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this OrmLLMModelConnection.


        :param type: The type of this OrmLLMModelConnection.  # noqa: E501
        :type type: str
        """

        self._type = type

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmLLMModelConnection.  # noqa: E501


        :return: The updated_dt of this OrmLLMModelConnection.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmLLMModelConnection.


        :param updated_dt: The updated_dt of this OrmLLMModelConnection.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def url(self):
        """Gets the url of this OrmLLMModelConnection.  # noqa: E501


        :return: The url of this OrmLLMModelConnection.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this OrmLLMModelConnection.


        :param url: The url of this OrmLLMModelConnection.  # noqa: E501
        :type url: str
        """

        self._url = url

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmLLMModelConnection):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmLLMModelConnection):
            return True

        return self.to_dict() != other.to_dict()
