# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class StorageCreateAPIInput(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cluster_id': 'int',
        'description': 'str',
        'name': 'str',
        'organization_credential_name': 'str',
        'read_only': 'bool',
        'storage_config': 'V1Config',
        'storage_type': 'str'
    }

    attribute_map = {
        'cluster_id': 'cluster_id',
        'description': 'description',
        'name': 'name',
        'organization_credential_name': 'organization_credential_name',
        'read_only': 'read_only',
        'storage_config': 'storage_config',
        'storage_type': 'storage_type'
    }

    def __init__(self, cluster_id=None, description=None, name=None, organization_credential_name=None, read_only=None, storage_config=None, storage_type=None, local_vars_configuration=None):  # noqa: E501
        """StorageCreateAPIInput - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cluster_id = None
        self._description = None
        self._name = None
        self._organization_credential_name = None
        self._read_only = None
        self._storage_config = None
        self._storage_type = None
        self.discriminator = None

        self.cluster_id = cluster_id
        self.description = description
        self.name = name
        self.organization_credential_name = organization_credential_name
        self.read_only = read_only
        self.storage_config = storage_config
        if storage_type is not None:
            self.storage_type = storage_type

    @property
    def cluster_id(self):
        """Gets the cluster_id of this StorageCreateAPIInput.  # noqa: E501


        :return: The cluster_id of this StorageCreateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this StorageCreateAPIInput.


        :param cluster_id: The cluster_id of this StorageCreateAPIInput.  # noqa: E501
        :type cluster_id: int
        """

        self._cluster_id = cluster_id

    @property
    def description(self):
        """Gets the description of this StorageCreateAPIInput.  # noqa: E501


        :return: The description of this StorageCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this StorageCreateAPIInput.


        :param description: The description of this StorageCreateAPIInput.  # noqa: E501
        :type description: str
        """
        if (self.local_vars_configuration.client_side_validation and
                description is not None and len(description) > 255):
            raise ValueError("Invalid value for `description`, length must be less than or equal to `255`")  # noqa: E501

        self._description = description

    @property
    def name(self):
        """Gets the name of this StorageCreateAPIInput.  # noqa: E501


        :return: The name of this StorageCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this StorageCreateAPIInput.


        :param name: The name of this StorageCreateAPIInput.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def organization_credential_name(self):
        """Gets the organization_credential_name of this StorageCreateAPIInput.  # noqa: E501


        :return: The organization_credential_name of this StorageCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._organization_credential_name

    @organization_credential_name.setter
    def organization_credential_name(self, organization_credential_name):
        """Sets the organization_credential_name of this StorageCreateAPIInput.


        :param organization_credential_name: The organization_credential_name of this StorageCreateAPIInput.  # noqa: E501
        :type organization_credential_name: str
        """

        self._organization_credential_name = organization_credential_name

    @property
    def read_only(self):
        """Gets the read_only of this StorageCreateAPIInput.  # noqa: E501


        :return: The read_only of this StorageCreateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._read_only

    @read_only.setter
    def read_only(self, read_only):
        """Sets the read_only of this StorageCreateAPIInput.


        :param read_only: The read_only of this StorageCreateAPIInput.  # noqa: E501
        :type read_only: bool
        """

        self._read_only = read_only

    @property
    def storage_config(self):
        """Gets the storage_config of this StorageCreateAPIInput.  # noqa: E501


        :return: The storage_config of this StorageCreateAPIInput.  # noqa: E501
        :rtype: V1Config
        """
        return self._storage_config

    @storage_config.setter
    def storage_config(self, storage_config):
        """Sets the storage_config of this StorageCreateAPIInput.


        :param storage_config: The storage_config of this StorageCreateAPIInput.  # noqa: E501
        :type storage_config: V1Config
        """
        if self.local_vars_configuration.client_side_validation and storage_config is None:  # noqa: E501
            raise ValueError("Invalid value for `storage_config`, must not be `None`")  # noqa: E501

        self._storage_config = storage_config

    @property
    def storage_type(self):
        """Gets the storage_type of this StorageCreateAPIInput.  # noqa: E501


        :return: The storage_type of this StorageCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._storage_type

    @storage_type.setter
    def storage_type(self, storage_type):
        """Sets the storage_type of this StorageCreateAPIInput.


        :param storage_type: The storage_type of this StorageCreateAPIInput.  # noqa: E501
        :type storage_type: str
        """
        allowed_values = ["s3", "gcs", "cluster-nfs", "cluster-host-path"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and storage_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `storage_type` ({0}), must be one of {1}"  # noqa: E501
                .format(storage_type, allowed_values)
            )

        self._storage_type = storage_type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StorageCreateAPIInput):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StorageCreateAPIInput):
            return True

        return self.to_dict() != other.to_dict()
