# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class V1HubModelTaskSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'category_id': 'int',
        'created_by': 'V1HubModelTaskSpecCreatedBy',
        'created_by_id': 'int',
        'description': 'str',
        'hide': 'bool',
        'hub_type': 'str',
        'id': 'int',
        'is_editable': 'bool',
        'key': 'str',
        'order_number': 'int',
        'overview': 'str',
        'public_url': 'str',
        'run_count': 'int',
        'scope': 'str',
        'status': 'str',
        'tags': 'list[V1HubTag]',
        'thumbnail_url': 'str',
        'title': 'str',
        'updated_dt': 'datetime',
        'version': 'int',
        'yaml': 'str'
    }

    attribute_map = {
        'category_id': 'category_id',
        'created_by': 'created_by',
        'created_by_id': 'created_by_id',
        'description': 'description',
        'hide': 'hide',
        'hub_type': 'hub_type',
        'id': 'id',
        'is_editable': 'is_editable',
        'key': 'key',
        'order_number': 'order_number',
        'overview': 'overview',
        'public_url': 'public_url',
        'run_count': 'run_count',
        'scope': 'scope',
        'status': 'status',
        'tags': 'tags',
        'thumbnail_url': 'thumbnail_url',
        'title': 'title',
        'updated_dt': 'updated_dt',
        'version': 'version',
        'yaml': 'yaml'
    }

    def __init__(self, category_id=None, created_by=None, created_by_id=None, description=None, hide=None, hub_type=None, id=None, is_editable=None, key=None, order_number=None, overview=None, public_url=None, run_count=None, scope=None, status=None, tags=None, thumbnail_url=None, title=None, updated_dt=None, version=None, yaml=None, local_vars_configuration=None):  # noqa: E501
        """V1HubModelTaskSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._category_id = None
        self._created_by = None
        self._created_by_id = None
        self._description = None
        self._hide = None
        self._hub_type = None
        self._id = None
        self._is_editable = None
        self._key = None
        self._order_number = None
        self._overview = None
        self._public_url = None
        self._run_count = None
        self._scope = None
        self._status = None
        self._tags = None
        self._thumbnail_url = None
        self._title = None
        self._updated_dt = None
        self._version = None
        self._yaml = None
        self.discriminator = None

        self.category_id = category_id
        if created_by is not None:
            self.created_by = created_by
        if created_by_id is not None:
            self.created_by_id = created_by_id
        if description is not None:
            self.description = description
        self.hide = hide
        self.hub_type = hub_type
        self.id = id
        self.is_editable = is_editable
        self.key = key
        self.order_number = order_number
        self.overview = overview
        if public_url is not None:
            self.public_url = public_url
        self.run_count = run_count
        self.scope = scope
        self.status = status
        if tags is not None:
            self.tags = tags
        if thumbnail_url is not None:
            self.thumbnail_url = thumbnail_url
        self.title = title
        self.updated_dt = updated_dt
        self.version = version
        self.yaml = yaml

    @property
    def category_id(self):
        """Gets the category_id of this V1HubModelTaskSpec.  # noqa: E501


        :return: The category_id of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: int
        """
        return self._category_id

    @category_id.setter
    def category_id(self, category_id):
        """Sets the category_id of this V1HubModelTaskSpec.


        :param category_id: The category_id of this V1HubModelTaskSpec.  # noqa: E501
        :type category_id: int
        """

        self._category_id = category_id

    @property
    def created_by(self):
        """Gets the created_by of this V1HubModelTaskSpec.  # noqa: E501


        :return: The created_by of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: V1HubModelTaskSpecCreatedBy
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this V1HubModelTaskSpec.


        :param created_by: The created_by of this V1HubModelTaskSpec.  # noqa: E501
        :type created_by: V1HubModelTaskSpecCreatedBy
        """

        self._created_by = created_by

    @property
    def created_by_id(self):
        """Gets the created_by_id of this V1HubModelTaskSpec.  # noqa: E501


        :return: The created_by_id of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: int
        """
        return self._created_by_id

    @created_by_id.setter
    def created_by_id(self, created_by_id):
        """Sets the created_by_id of this V1HubModelTaskSpec.


        :param created_by_id: The created_by_id of this V1HubModelTaskSpec.  # noqa: E501
        :type created_by_id: int
        """

        self._created_by_id = created_by_id

    @property
    def description(self):
        """Gets the description of this V1HubModelTaskSpec.  # noqa: E501


        :return: The description of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this V1HubModelTaskSpec.


        :param description: The description of this V1HubModelTaskSpec.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def hide(self):
        """Gets the hide of this V1HubModelTaskSpec.  # noqa: E501


        :return: The hide of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: bool
        """
        return self._hide

    @hide.setter
    def hide(self, hide):
        """Sets the hide of this V1HubModelTaskSpec.


        :param hide: The hide of this V1HubModelTaskSpec.  # noqa: E501
        :type hide: bool
        """
        if self.local_vars_configuration.client_side_validation and hide is None:  # noqa: E501
            raise ValueError("Invalid value for `hide`, must not be `None`")  # noqa: E501

        self._hide = hide

    @property
    def hub_type(self):
        """Gets the hub_type of this V1HubModelTaskSpec.  # noqa: E501


        :return: The hub_type of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._hub_type

    @hub_type.setter
    def hub_type(self, hub_type):
        """Sets the hub_type of this V1HubModelTaskSpec.


        :param hub_type: The hub_type of this V1HubModelTaskSpec.  # noqa: E501
        :type hub_type: str
        """
        if self.local_vars_configuration.client_side_validation and hub_type is None:  # noqa: E501
            raise ValueError("Invalid value for `hub_type`, must not be `None`")  # noqa: E501

        self._hub_type = hub_type

    @property
    def id(self):
        """Gets the id of this V1HubModelTaskSpec.  # noqa: E501


        :return: The id of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this V1HubModelTaskSpec.


        :param id: The id of this V1HubModelTaskSpec.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def is_editable(self):
        """Gets the is_editable of this V1HubModelTaskSpec.  # noqa: E501


        :return: The is_editable of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: bool
        """
        return self._is_editable

    @is_editable.setter
    def is_editable(self, is_editable):
        """Sets the is_editable of this V1HubModelTaskSpec.


        :param is_editable: The is_editable of this V1HubModelTaskSpec.  # noqa: E501
        :type is_editable: bool
        """
        if self.local_vars_configuration.client_side_validation and is_editable is None:  # noqa: E501
            raise ValueError("Invalid value for `is_editable`, must not be `None`")  # noqa: E501

        self._is_editable = is_editable

    @property
    def key(self):
        """Gets the key of this V1HubModelTaskSpec.  # noqa: E501


        :return: The key of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this V1HubModelTaskSpec.


        :param key: The key of this V1HubModelTaskSpec.  # noqa: E501
        :type key: str
        """
        if self.local_vars_configuration.client_side_validation and key is None:  # noqa: E501
            raise ValueError("Invalid value for `key`, must not be `None`")  # noqa: E501

        self._key = key

    @property
    def order_number(self):
        """Gets the order_number of this V1HubModelTaskSpec.  # noqa: E501


        :return: The order_number of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: int
        """
        return self._order_number

    @order_number.setter
    def order_number(self, order_number):
        """Sets the order_number of this V1HubModelTaskSpec.


        :param order_number: The order_number of this V1HubModelTaskSpec.  # noqa: E501
        :type order_number: int
        """
        if self.local_vars_configuration.client_side_validation and order_number is None:  # noqa: E501
            raise ValueError("Invalid value for `order_number`, must not be `None`")  # noqa: E501

        self._order_number = order_number

    @property
    def overview(self):
        """Gets the overview of this V1HubModelTaskSpec.  # noqa: E501


        :return: The overview of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._overview

    @overview.setter
    def overview(self, overview):
        """Sets the overview of this V1HubModelTaskSpec.


        :param overview: The overview of this V1HubModelTaskSpec.  # noqa: E501
        :type overview: str
        """
        if self.local_vars_configuration.client_side_validation and overview is None:  # noqa: E501
            raise ValueError("Invalid value for `overview`, must not be `None`")  # noqa: E501

        self._overview = overview

    @property
    def public_url(self):
        """Gets the public_url of this V1HubModelTaskSpec.  # noqa: E501


        :return: The public_url of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._public_url

    @public_url.setter
    def public_url(self, public_url):
        """Sets the public_url of this V1HubModelTaskSpec.


        :param public_url: The public_url of this V1HubModelTaskSpec.  # noqa: E501
        :type public_url: str
        """

        self._public_url = public_url

    @property
    def run_count(self):
        """Gets the run_count of this V1HubModelTaskSpec.  # noqa: E501


        :return: The run_count of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: int
        """
        return self._run_count

    @run_count.setter
    def run_count(self, run_count):
        """Sets the run_count of this V1HubModelTaskSpec.


        :param run_count: The run_count of this V1HubModelTaskSpec.  # noqa: E501
        :type run_count: int
        """
        if self.local_vars_configuration.client_side_validation and run_count is None:  # noqa: E501
            raise ValueError("Invalid value for `run_count`, must not be `None`")  # noqa: E501

        self._run_count = run_count

    @property
    def scope(self):
        """Gets the scope of this V1HubModelTaskSpec.  # noqa: E501


        :return: The scope of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._scope

    @scope.setter
    def scope(self, scope):
        """Sets the scope of this V1HubModelTaskSpec.


        :param scope: The scope of this V1HubModelTaskSpec.  # noqa: E501
        :type scope: str
        """
        if self.local_vars_configuration.client_side_validation and scope is None:  # noqa: E501
            raise ValueError("Invalid value for `scope`, must not be `None`")  # noqa: E501

        self._scope = scope

    @property
    def status(self):
        """Gets the status of this V1HubModelTaskSpec.  # noqa: E501


        :return: The status of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this V1HubModelTaskSpec.


        :param status: The status of this V1HubModelTaskSpec.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def tags(self):
        """Gets the tags of this V1HubModelTaskSpec.  # noqa: E501


        :return: The tags of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: list[V1HubTag]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this V1HubModelTaskSpec.


        :param tags: The tags of this V1HubModelTaskSpec.  # noqa: E501
        :type tags: list[V1HubTag]
        """

        self._tags = tags

    @property
    def thumbnail_url(self):
        """Gets the thumbnail_url of this V1HubModelTaskSpec.  # noqa: E501


        :return: The thumbnail_url of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._thumbnail_url

    @thumbnail_url.setter
    def thumbnail_url(self, thumbnail_url):
        """Sets the thumbnail_url of this V1HubModelTaskSpec.


        :param thumbnail_url: The thumbnail_url of this V1HubModelTaskSpec.  # noqa: E501
        :type thumbnail_url: str
        """

        self._thumbnail_url = thumbnail_url

    @property
    def title(self):
        """Gets the title of this V1HubModelTaskSpec.  # noqa: E501


        :return: The title of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this V1HubModelTaskSpec.


        :param title: The title of this V1HubModelTaskSpec.  # noqa: E501
        :type title: str
        """
        if self.local_vars_configuration.client_side_validation and title is None:  # noqa: E501
            raise ValueError("Invalid value for `title`, must not be `None`")  # noqa: E501

        self._title = title

    @property
    def updated_dt(self):
        """Gets the updated_dt of this V1HubModelTaskSpec.  # noqa: E501


        :return: The updated_dt of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this V1HubModelTaskSpec.


        :param updated_dt: The updated_dt of this V1HubModelTaskSpec.  # noqa: E501
        :type updated_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_dt`, must not be `None`")  # noqa: E501

        self._updated_dt = updated_dt

    @property
    def version(self):
        """Gets the version of this V1HubModelTaskSpec.  # noqa: E501


        :return: The version of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this V1HubModelTaskSpec.


        :param version: The version of this V1HubModelTaskSpec.  # noqa: E501
        :type version: int
        """

        self._version = version

    @property
    def yaml(self):
        """Gets the yaml of this V1HubModelTaskSpec.  # noqa: E501


        :return: The yaml of this V1HubModelTaskSpec.  # noqa: E501
        :rtype: str
        """
        return self._yaml

    @yaml.setter
    def yaml(self, yaml):
        """Sets the yaml of this V1HubModelTaskSpec.


        :param yaml: The yaml of this V1HubModelTaskSpec.  # noqa: E501
        :type yaml: str
        """
        if self.local_vars_configuration.client_side_validation and yaml is None:  # noqa: E501
            raise ValueError("Invalid value for `yaml`, must not be `None`")  # noqa: E501

        self._yaml = yaml

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1HubModelTaskSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1HubModelTaskSpec):
            return True

        return self.to_dict() != other.to_dict()
