# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class ProjectNotificationSettingsUpdateAPIInput(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'enabled': 'bool',
        'message_template': 'str',
        'on_run_complete': 'bool',
        'on_run_failure': 'bool',
        'on_run_start': 'bool'
    }

    attribute_map = {
        'enabled': 'enabled',
        'message_template': 'message_template',
        'on_run_complete': 'on_run_complete',
        'on_run_failure': 'on_run_failure',
        'on_run_start': 'on_run_start'
    }

    def __init__(self, enabled=None, message_template=None, on_run_complete=None, on_run_failure=None, on_run_start=None, local_vars_configuration=None):  # noqa: E501
        """ProjectNotificationSettingsUpdateAPIInput - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._enabled = None
        self._message_template = None
        self._on_run_complete = None
        self._on_run_failure = None
        self._on_run_start = None
        self.discriminator = None

        if enabled is not None:
            self.enabled = enabled
        if message_template is not None:
            self.message_template = message_template
        if on_run_complete is not None:
            self.on_run_complete = on_run_complete
        if on_run_failure is not None:
            self.on_run_failure = on_run_failure
        if on_run_start is not None:
            self.on_run_start = on_run_start

    @property
    def enabled(self):
        """Gets the enabled of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The enabled of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this ProjectNotificationSettingsUpdateAPIInput.


        :param enabled: The enabled of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type enabled: bool
        """

        self._enabled = enabled

    @property
    def message_template(self):
        """Gets the message_template of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The message_template of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._message_template

    @message_template.setter
    def message_template(self, message_template):
        """Sets the message_template of this ProjectNotificationSettingsUpdateAPIInput.


        :param message_template: The message_template of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type message_template: str
        """

        self._message_template = message_template

    @property
    def on_run_complete(self):
        """Gets the on_run_complete of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The on_run_complete of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._on_run_complete

    @on_run_complete.setter
    def on_run_complete(self, on_run_complete):
        """Sets the on_run_complete of this ProjectNotificationSettingsUpdateAPIInput.


        :param on_run_complete: The on_run_complete of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type on_run_complete: bool
        """

        self._on_run_complete = on_run_complete

    @property
    def on_run_failure(self):
        """Gets the on_run_failure of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The on_run_failure of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._on_run_failure

    @on_run_failure.setter
    def on_run_failure(self, on_run_failure):
        """Sets the on_run_failure of this ProjectNotificationSettingsUpdateAPIInput.


        :param on_run_failure: The on_run_failure of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type on_run_failure: bool
        """

        self._on_run_failure = on_run_failure

    @property
    def on_run_start(self):
        """Gets the on_run_start of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501


        :return: The on_run_start of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._on_run_start

    @on_run_start.setter
    def on_run_start(self, on_run_start):
        """Sets the on_run_start of this ProjectNotificationSettingsUpdateAPIInput.


        :param on_run_start: The on_run_start of this ProjectNotificationSettingsUpdateAPIInput.  # noqa: E501
        :type on_run_start: bool
        """

        self._on_run_start = on_run_start

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProjectNotificationSettingsUpdateAPIInput):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ProjectNotificationSettingsUpdateAPIInput):
            return True

        return self.to_dict() != other.to_dict()
