# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmModelEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'artifact': 'OrmArtifact',
        'artifact_volume': 'OrmVolume',
        'created_by': 'OrmUser',
        'experiment': 'OrmExperiment',
        'model_generated_experiments': 'list[OrmModelGeneratedExperiment]',
        'model_repository': 'OrmModelRepository',
        'tags': 'list[OrmTag]'
    }

    attribute_map = {
        'artifact': 'artifact',
        'artifact_volume': 'artifact_volume',
        'created_by': 'created_by',
        'experiment': 'experiment',
        'model_generated_experiments': 'model_generated_experiments',
        'model_repository': 'model_repository',
        'tags': 'tags'
    }

    def __init__(self, artifact=None, artifact_volume=None, created_by=None, experiment=None, model_generated_experiments=None, model_repository=None, tags=None, local_vars_configuration=None):  # noqa: E501
        """OrmModelEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._artifact = None
        self._artifact_volume = None
        self._created_by = None
        self._experiment = None
        self._model_generated_experiments = None
        self._model_repository = None
        self._tags = None
        self.discriminator = None

        if artifact is not None:
            self.artifact = artifact
        if artifact_volume is not None:
            self.artifact_volume = artifact_volume
        if created_by is not None:
            self.created_by = created_by
        if experiment is not None:
            self.experiment = experiment
        if model_generated_experiments is not None:
            self.model_generated_experiments = model_generated_experiments
        if model_repository is not None:
            self.model_repository = model_repository
        if tags is not None:
            self.tags = tags

    @property
    def artifact(self):
        """Gets the artifact of this OrmModelEdges.  # noqa: E501


        :return: The artifact of this OrmModelEdges.  # noqa: E501
        :rtype: OrmArtifact
        """
        return self._artifact

    @artifact.setter
    def artifact(self, artifact):
        """Sets the artifact of this OrmModelEdges.


        :param artifact: The artifact of this OrmModelEdges.  # noqa: E501
        :type artifact: OrmArtifact
        """

        self._artifact = artifact

    @property
    def artifact_volume(self):
        """Gets the artifact_volume of this OrmModelEdges.  # noqa: E501


        :return: The artifact_volume of this OrmModelEdges.  # noqa: E501
        :rtype: OrmVolume
        """
        return self._artifact_volume

    @artifact_volume.setter
    def artifact_volume(self, artifact_volume):
        """Sets the artifact_volume of this OrmModelEdges.


        :param artifact_volume: The artifact_volume of this OrmModelEdges.  # noqa: E501
        :type artifact_volume: OrmVolume
        """

        self._artifact_volume = artifact_volume

    @property
    def created_by(self):
        """Gets the created_by of this OrmModelEdges.  # noqa: E501


        :return: The created_by of this OrmModelEdges.  # noqa: E501
        :rtype: OrmUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this OrmModelEdges.


        :param created_by: The created_by of this OrmModelEdges.  # noqa: E501
        :type created_by: OrmUser
        """

        self._created_by = created_by

    @property
    def experiment(self):
        """Gets the experiment of this OrmModelEdges.  # noqa: E501


        :return: The experiment of this OrmModelEdges.  # noqa: E501
        :rtype: OrmExperiment
        """
        return self._experiment

    @experiment.setter
    def experiment(self, experiment):
        """Sets the experiment of this OrmModelEdges.


        :param experiment: The experiment of this OrmModelEdges.  # noqa: E501
        :type experiment: OrmExperiment
        """

        self._experiment = experiment

    @property
    def model_generated_experiments(self):
        """Gets the model_generated_experiments of this OrmModelEdges.  # noqa: E501


        :return: The model_generated_experiments of this OrmModelEdges.  # noqa: E501
        :rtype: list[OrmModelGeneratedExperiment]
        """
        return self._model_generated_experiments

    @model_generated_experiments.setter
    def model_generated_experiments(self, model_generated_experiments):
        """Sets the model_generated_experiments of this OrmModelEdges.


        :param model_generated_experiments: The model_generated_experiments of this OrmModelEdges.  # noqa: E501
        :type model_generated_experiments: list[OrmModelGeneratedExperiment]
        """

        self._model_generated_experiments = model_generated_experiments

    @property
    def model_repository(self):
        """Gets the model_repository of this OrmModelEdges.  # noqa: E501


        :return: The model_repository of this OrmModelEdges.  # noqa: E501
        :rtype: OrmModelRepository
        """
        return self._model_repository

    @model_repository.setter
    def model_repository(self, model_repository):
        """Sets the model_repository of this OrmModelEdges.


        :param model_repository: The model_repository of this OrmModelEdges.  # noqa: E501
        :type model_repository: OrmModelRepository
        """

        self._model_repository = model_repository

    @property
    def tags(self):
        """Gets the tags of this OrmModelEdges.  # noqa: E501


        :return: The tags of this OrmModelEdges.  # noqa: E501
        :rtype: list[OrmTag]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this OrmModelEdges.


        :param tags: The tags of this OrmModelEdges.  # noqa: E501
        :type tags: list[OrmTag]
        """

        self._tags = tags

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmModelEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmModelEdges):
            return True

        return self.to_dict() != other.to_dict()
