# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class ProtoLLMWorkflow(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_by': 'ProtoUser',
        'created_by_id': 'int',
        'created_dt': 'float',
        'default_resource_preset_id': 'int',
        'description': 'str',
        'id': 'int',
        'model_service_id': 'int',
        'name': 'str',
        'permission_config': 'V1LLMPermissionConfig',
        'status': 'str',
        'updated_dt': 'float'
    }

    attribute_map = {
        'created_by': 'created_by',
        'created_by_id': 'created_by_id',
        'created_dt': 'created_dt',
        'default_resource_preset_id': 'default_resource_preset_id',
        'description': 'description',
        'id': 'id',
        'model_service_id': 'model_service_id',
        'name': 'name',
        'permission_config': 'permission_config',
        'status': 'status',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, created_by=None, created_by_id=None, created_dt=None, default_resource_preset_id=None, description=None, id=None, model_service_id=None, name=None, permission_config=None, status=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """ProtoLLMWorkflow - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_by = None
        self._created_by_id = None
        self._created_dt = None
        self._default_resource_preset_id = None
        self._description = None
        self._id = None
        self._model_service_id = None
        self._name = None
        self._permission_config = None
        self._status = None
        self._updated_dt = None
        self.discriminator = None

        if created_by is not None:
            self.created_by = created_by
        self.created_by_id = created_by_id
        self.created_dt = created_dt
        self.default_resource_preset_id = default_resource_preset_id
        if description is not None:
            self.description = description
        self.id = id
        self.model_service_id = model_service_id
        self.name = name
        if permission_config is not None:
            self.permission_config = permission_config
        self.status = status
        self.updated_dt = updated_dt

    @property
    def created_by(self):
        """Gets the created_by of this ProtoLLMWorkflow.  # noqa: E501


        :return: The created_by of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: ProtoUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ProtoLLMWorkflow.


        :param created_by: The created_by of this ProtoLLMWorkflow.  # noqa: E501
        :type created_by: ProtoUser
        """

        self._created_by = created_by

    @property
    def created_by_id(self):
        """Gets the created_by_id of this ProtoLLMWorkflow.  # noqa: E501


        :return: The created_by_id of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: int
        """
        return self._created_by_id

    @created_by_id.setter
    def created_by_id(self, created_by_id):
        """Sets the created_by_id of this ProtoLLMWorkflow.


        :param created_by_id: The created_by_id of this ProtoLLMWorkflow.  # noqa: E501
        :type created_by_id: int
        """
        if self.local_vars_configuration.client_side_validation and created_by_id is None:  # noqa: E501
            raise ValueError("Invalid value for `created_by_id`, must not be `None`")  # noqa: E501

        self._created_by_id = created_by_id

    @property
    def created_dt(self):
        """Gets the created_dt of this ProtoLLMWorkflow.  # noqa: E501


        :return: The created_dt of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: float
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ProtoLLMWorkflow.


        :param created_dt: The created_dt of this ProtoLLMWorkflow.  # noqa: E501
        :type created_dt: float
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def default_resource_preset_id(self):
        """Gets the default_resource_preset_id of this ProtoLLMWorkflow.  # noqa: E501


        :return: The default_resource_preset_id of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: int
        """
        return self._default_resource_preset_id

    @default_resource_preset_id.setter
    def default_resource_preset_id(self, default_resource_preset_id):
        """Sets the default_resource_preset_id of this ProtoLLMWorkflow.


        :param default_resource_preset_id: The default_resource_preset_id of this ProtoLLMWorkflow.  # noqa: E501
        :type default_resource_preset_id: int
        """
        if self.local_vars_configuration.client_side_validation and default_resource_preset_id is None:  # noqa: E501
            raise ValueError("Invalid value for `default_resource_preset_id`, must not be `None`")  # noqa: E501

        self._default_resource_preset_id = default_resource_preset_id

    @property
    def description(self):
        """Gets the description of this ProtoLLMWorkflow.  # noqa: E501


        :return: The description of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ProtoLLMWorkflow.


        :param description: The description of this ProtoLLMWorkflow.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def id(self):
        """Gets the id of this ProtoLLMWorkflow.  # noqa: E501


        :return: The id of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ProtoLLMWorkflow.


        :param id: The id of this ProtoLLMWorkflow.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def model_service_id(self):
        """Gets the model_service_id of this ProtoLLMWorkflow.  # noqa: E501


        :return: The model_service_id of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: int
        """
        return self._model_service_id

    @model_service_id.setter
    def model_service_id(self, model_service_id):
        """Sets the model_service_id of this ProtoLLMWorkflow.


        :param model_service_id: The model_service_id of this ProtoLLMWorkflow.  # noqa: E501
        :type model_service_id: int
        """
        if self.local_vars_configuration.client_side_validation and model_service_id is None:  # noqa: E501
            raise ValueError("Invalid value for `model_service_id`, must not be `None`")  # noqa: E501

        self._model_service_id = model_service_id

    @property
    def name(self):
        """Gets the name of this ProtoLLMWorkflow.  # noqa: E501


        :return: The name of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ProtoLLMWorkflow.


        :param name: The name of this ProtoLLMWorkflow.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def permission_config(self):
        """Gets the permission_config of this ProtoLLMWorkflow.  # noqa: E501


        :return: The permission_config of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: V1LLMPermissionConfig
        """
        return self._permission_config

    @permission_config.setter
    def permission_config(self, permission_config):
        """Sets the permission_config of this ProtoLLMWorkflow.


        :param permission_config: The permission_config of this ProtoLLMWorkflow.  # noqa: E501
        :type permission_config: V1LLMPermissionConfig
        """

        self._permission_config = permission_config

    @property
    def status(self):
        """Gets the status of this ProtoLLMWorkflow.  # noqa: E501


        :return: The status of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ProtoLLMWorkflow.


        :param status: The status of this ProtoLLMWorkflow.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501
        allowed_values = ["idle", "updating", "live"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and status not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}"  # noqa: E501
                .format(status, allowed_values)
            )

        self._status = status

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ProtoLLMWorkflow.  # noqa: E501


        :return: The updated_dt of this ProtoLLMWorkflow.  # noqa: E501
        :rtype: float
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ProtoLLMWorkflow.


        :param updated_dt: The updated_dt of this ProtoLLMWorkflow.  # noqa: E501
        :type updated_dt: float
        """
        if self.local_vars_configuration.client_side_validation and updated_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_dt`, must not be `None`")  # noqa: E501

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProtoLLMWorkflow):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ProtoLLMWorkflow):
            return True

        return self.to_dict() != other.to_dict()
