# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class PrometheusqueryLatestNodeMetricSample(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cpu': 'PrometheusqueryNodeResourceMetricSample',
        'disk': 'PrometheusqueryNodeResourceMetricSample',
        'gpu': 'PrometheusqueryNodeResourceMetricSample',
        'gpu_slice': 'PrometheusqueryNodeResourceMetricSample',
        'memory': 'PrometheusqueryNodeResourceMetricSample'
    }

    attribute_map = {
        'cpu': 'cpu',
        'disk': 'disk',
        'gpu': 'gpu',
        'gpu_slice': 'gpu_slice',
        'memory': 'memory'
    }

    def __init__(self, cpu=None, disk=None, gpu=None, gpu_slice=None, memory=None, local_vars_configuration=None):  # noqa: E501
        """PrometheusqueryLatestNodeMetricSample - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cpu = None
        self._disk = None
        self._gpu = None
        self._gpu_slice = None
        self._memory = None
        self.discriminator = None

        self.cpu = cpu
        self.disk = disk
        self.gpu = gpu
        self.gpu_slice = gpu_slice
        self.memory = memory

    @property
    def cpu(self):
        """Gets the cpu of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501


        :return: The cpu of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :rtype: PrometheusqueryNodeResourceMetricSample
        """
        return self._cpu

    @cpu.setter
    def cpu(self, cpu):
        """Sets the cpu of this PrometheusqueryLatestNodeMetricSample.


        :param cpu: The cpu of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :type cpu: PrometheusqueryNodeResourceMetricSample
        """
        if self.local_vars_configuration.client_side_validation and cpu is None:  # noqa: E501
            raise ValueError("Invalid value for `cpu`, must not be `None`")  # noqa: E501

        self._cpu = cpu

    @property
    def disk(self):
        """Gets the disk of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501


        :return: The disk of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :rtype: PrometheusqueryNodeResourceMetricSample
        """
        return self._disk

    @disk.setter
    def disk(self, disk):
        """Sets the disk of this PrometheusqueryLatestNodeMetricSample.


        :param disk: The disk of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :type disk: PrometheusqueryNodeResourceMetricSample
        """
        if self.local_vars_configuration.client_side_validation and disk is None:  # noqa: E501
            raise ValueError("Invalid value for `disk`, must not be `None`")  # noqa: E501

        self._disk = disk

    @property
    def gpu(self):
        """Gets the gpu of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501


        :return: The gpu of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :rtype: PrometheusqueryNodeResourceMetricSample
        """
        return self._gpu

    @gpu.setter
    def gpu(self, gpu):
        """Sets the gpu of this PrometheusqueryLatestNodeMetricSample.


        :param gpu: The gpu of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :type gpu: PrometheusqueryNodeResourceMetricSample
        """
        if self.local_vars_configuration.client_side_validation and gpu is None:  # noqa: E501
            raise ValueError("Invalid value for `gpu`, must not be `None`")  # noqa: E501

        self._gpu = gpu

    @property
    def gpu_slice(self):
        """Gets the gpu_slice of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501


        :return: The gpu_slice of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :rtype: PrometheusqueryNodeResourceMetricSample
        """
        return self._gpu_slice

    @gpu_slice.setter
    def gpu_slice(self, gpu_slice):
        """Sets the gpu_slice of this PrometheusqueryLatestNodeMetricSample.


        :param gpu_slice: The gpu_slice of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :type gpu_slice: PrometheusqueryNodeResourceMetricSample
        """
        if self.local_vars_configuration.client_side_validation and gpu_slice is None:  # noqa: E501
            raise ValueError("Invalid value for `gpu_slice`, must not be `None`")  # noqa: E501

        self._gpu_slice = gpu_slice

    @property
    def memory(self):
        """Gets the memory of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501


        :return: The memory of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :rtype: PrometheusqueryNodeResourceMetricSample
        """
        return self._memory

    @memory.setter
    def memory(self, memory):
        """Sets the memory of this PrometheusqueryLatestNodeMetricSample.


        :param memory: The memory of this PrometheusqueryLatestNodeMetricSample.  # noqa: E501
        :type memory: PrometheusqueryNodeResourceMetricSample
        """
        if self.local_vars_configuration.client_side_validation and memory is None:  # noqa: E501
            raise ValueError("Invalid value for `memory`, must not be `None`")  # noqa: E501

        self._memory = memory

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PrometheusqueryLatestNodeMetricSample):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, PrometheusqueryLatestNodeMetricSample):
            return True

        return self.to_dict() != other.to_dict()
